/********
        *       Includes for BOINC
********/
#ifdef _WIN32
#include "boinc_win.h"
#else
#include "config.h"
#endif

#ifndef _WIN32
#include <cstdio>
#include <cctype>
#include <ctime>
#include <cstring>
#include <cstdlib>
#include <csignal>
#include <unistd.h>
#endif

/* #define BOINC_APP_GRAPHICS */

#ifdef BOINC_APP_GRAPHICS
#include "graphics_api.h"
#include "graphics_lib.h"
#endif

#include "diagnostics.h"
#include "str_util.h"
#include "util.h"
#include "filesys.h"
#include "boinc_api.h"
#include "mfile.h"
#include <stdio.h>

using std::string;

#define CHECKPOINT_FILE "astronomy_state"
#define OUTPUT_FILENAME "out"

/********
	*	Includes for astronomy
 ********/
#include "parameters.h"
#include "volume.h"
#include "star_points.h"
#include "integral_function.h"
#include "likelihood_function.h"


int integral_results_length;
int likelihood_parameter_length;
int likelihood_results_length;


void worker() {
	double* likelihood_result;
	double	background_integral;
	double*	stream_integrals;

    // variables for BOINC
    int retval;
    char output_path[512];
    MFILE out;

        // read the chkpts
	read_parameters();
	read_likelihood_data(0,1);
	read_integral_data(0,1);
	
    // open output file
    boinc_resolve_filename(OUTPUT_FILENAME, output_path, sizeof(output_path));

    integral_results_length = get_number_streams() + 1;
    likelihood_parameter_length = get_number_streams() + 1;
    likelihood_results_length = 1;

    calculate_integrals();
    background_integral = get_background_integral();
    stream_integrals = get_stream_integrals();

	likelihood_result = (double*)malloc(sizeof(double));
	likelihood_result[0] = calculate_likelihood(background_integral, stream_integrals);

	out.open(output_path, "w");

	out.printf("calculated likelihood: %lf\n", likelihood_result[0]);

	retval = out.flush();
	if (retval) {
        fprintf(stderr, "APP: astronomy flush failed %d\n", retval);
        exit(1);
	}
	out.close();

	free(likelihood_result);
	free(stream_integrals);
	parameter_dump();
	star_point_dump();

#ifdef _WIN32
	_set_printf_count_output( 1 );
	_get_printf_count_output();
#endif

	boinc_finish(0);
}

int main(int argc, char **argv){
        int retval = 0;

#ifdef BOINC_APP_GRAPHICS
#if defined(_WIN32) || defined(__APPLE__)
	        retval = boinc_init_graphics(worker);
#else
                retval = boinc_init_graphics_lib(worker, argv[0]);
#endif
                if (retval) exit(retval);
#endif

	retval = boinc_init();
        if (retval) exit(retval);
        worker();
}

#ifdef _WIN32
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR Args, int WinMode){
        LPSTR command_line;
        char* argv[100];
	int argc;

	command_line = GetCommandLine();
	argc = parse_command_line( command_line, argv );
	return main(argc, argv);
}
#endif

const char *BOINC_RCSID_33ac47a071 = "$Id: astronomy.c,v 1.12 2007/10/07 05:45:49 doranj Exp $";
