/*
 *  star_points.c
 *  Astronomy
 *
 *  Created by Travis Desell on 2/21/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

/****
         *     BOINC includes
*****/

#ifdef _WIN32
#include "boinc_win.h"
#else
#include "config.h"
#endif

#ifndef _WIN32
#include <cstdio>
#include <cctype>
#include <ctime>
#include <cstring>
#include <cstdlib>
#include <csignal>
#include <unistd.h>
#endif

#define BOINC_APP_GRAPHICS

#ifdef BOINC_APP_GRAPHICS
#include "graphics_api.h"
#include "graphics_lib.h"
#endif

#include "diagnostics.h"
#include "str_util.h"
#include "util.h"
#include "filesys.h"
#include "boinc_api.h"
#include "mfile.h"

using std::string;

#define INPUT_FILENAME "stars.txt"

/****
	*	Astronomy includes
*****/
#include "star_points.h"


/****
	*	Star point variables
*****/
long number_star_points;
long total_star_points;
double **star_points;

long		get_total_star_points()		{ return total_star_points; }
long		get_number_star_points()	{ return number_star_points; }
double**	get_star_points()		{ return star_points; }

void read_likelihood_data(int rank, int slices) {
	long slice_min, slice_max, current_slice;
	int i, j;
	double slice_size;
	double** temp_data;

	read_star_points();

	slice_size = (double)number_star_points / (double)slices;
	slice_min = 0;
	for (i = 0; i < slices; i++) {
		if (i == slices-1) {
			slice_max = number_star_points;
		} else {
			slice_max = (long)((double)(i+1) * (slice_size));
		}

		if (i == rank) {
			current_slice = slice_max-slice_min;

			temp_data = (double**)malloc(sizeof(double*) * current_slice);
			for (j = 0; j < current_slice; j++) {
				temp_data[j] = (double*)malloc(sizeof(double) * 3);
				temp_data[j][0] = star_points[slice_min + j][0];
				temp_data[j][1] = star_points[slice_min + j][1];
				temp_data[j][2] = star_points[slice_min + j][2];
				free(star_points[slice_min + j]);
			}
			for (j = 0; j < slice_min; j++)
				free(star_points[j]);
			for (j = slice_max; j < number_star_points; j++)
				free(star_points[j]);
			free(star_points);
//			fprintf(stdout, "splitting likelihood data, min: %ld, max: %ld, size: %ld\n", slice_min, slice_max, current_slice);
			star_points = temp_data;
			number_star_points = current_slice;
			return;
		}
		slice_min = slice_max;
	}
}

void read_star_points() {
	FILE *data_file;
	int i;
	char input_path[512];
	
	boinc_resolve_filename(INPUT_FILENAME, input_path, sizeof(input_path));
	data_file = boinc_fopen(input_path, "r");
	if (!data_file) {
		fprintf(stderr, "Couldn't find input file %s.\n", INPUT_FILENAME);
		exit(-1);
	}
//	fprintf(stderr, "opened file: %s\n", INPUT_FILENAME);

	fscanf(data_file, "%d\n", &number_star_points);
//	fprintf(stderr, "reading %d star points.\n", number_star_points);

	total_star_points = number_star_points;
	star_points = (double**)malloc(sizeof(double*)*number_star_points);
	for (i = 0; i < number_star_points; i++) {
		star_points[i] = (double*)malloc(sizeof(double)*3);
		fscanf(data_file, "%lf %lf %lf\n", &star_points[i][0], &star_points[i][1], &star_points[i][2]);
//		if (i%10000 == 0) fprintf(stderr, "read %d points. last: %lf %lf %lf\n", i, star_points[i][0], star_points[i][1], star_points[i][2]);
	}

//	fprintf(stderr, "finished reading %d star points.\n", number_star_points);
//	fprintf(stderr, "completed read_star_data.\n\n");
}

void star_point_dump() {
	int i;

	for(i = 0; i < number_star_points; i++) {
		free(star_points[i]);
	}

	free(star_points);
}
