/*
 *  volume.c
 *  Astronomy
 *
 *  Created by Travis Desell on 2/21/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

/****
        *       BOINC includes
 ****/
#ifdef _WIN32
#include "boinc_win.h"
#else
#include "config.h"
#endif

#ifndef _WIN32
#include <cstdio>
#include <cctype>
#include <ctime>
#include <cstring>
#include <cstdlib>
#include <csignal>
#include <unistd.h>
#endif

#define BOINC_APP_GRAPHICS

#ifdef BOINC_APP_GRAPHICS
#include "graphics_api.h"
#include "graphics_lib.h"
#endif

#include "diagnostics.h"
#include "str_util.h"
#include "util.h"
#include "filesys.h"
#include "boinc_api.h"
#include "mfile.h"

using std::string;

#define INPUT_FILENAME "volume.txt"

/****
	*	Astronomy includes
*****/
#include "volume.h"


/****
	*	Volume variables
*****/
#define VOLUME_FILE "volume.txt"
long wedge;
double r_min, r_max, r_step;
long r_steps;
double nu_min, nu_max, nu_step;
long nu_steps;
double mu_min, mu_max, mu_step;
long mu_steps;

#define num_limits 27
int limits[27][3] = {{ 9, 122, 238 }, { 10, 117, 249 }, { 11, 124, 242 }, { 12, 120, 240 }, { 13, 120, 244 },
					{ 14, 123, 217 }, { 15, 120, 252 }, { 16, 119, 194 }, { 27, 120, 164 }, { 28, 119, 164 },
					{ 29, 122, 187 }, { 30, 121, 222 }, { 31, 122, 204 }, { 32, 125, 248 }, { 33, 124, 238 },
					{ 34, 125, 246 }, { 35, 128, 247 }, { 36, 128, 246 }, { 37, 132, 247 }, { 42, 208, 220 },
					{ 43, 206, 235 }, { 44, 194, 228 }, { 76, 308, 392 }, { 82, 307, 436 }, { 83, 307, 418 },
					{ 86, 307, 426 }, { -1, 0, 0 } };

/****
	*	Functions to access the volume
*****/
int	get_wedge()	{ return wedge; }

double	get_r_min()	{ return r_min; }
double	get_r_max()	{ return r_max; }
double	get_r_step()	{ return r_step; }
double	get_r_steps()	{ return r_steps; }

double	get_mu_min()	{ return mu_min; }
double	get_mu_max()	{ return mu_max; }
double	get_mu_step()	{ return mu_step; }
double	get_mu_steps()	{ return mu_steps; }

double	get_nu_min()	{ return nu_min; }
double	get_nu_max()	{ return nu_max; }
double	get_nu_step()	{ return nu_step; }
double	get_nu_steps()	{ return nu_steps; }

/********
	*	Read volume from a file
*********/
void read_integral_data(int rank, int slices) {
	double r_min_step, r_max_step;
	char input_path[512];
	FILE *volume_file;
	int i;
	
	fprintf(stderr, "APP: astronomy reading volume from file: %s\n", VOLUME_FILE);
	boinc_resolve_filename(INPUT_FILENAME, input_path, sizeof(input_path));
	volume_file = boinc_fopen(input_path, "r");
	if (!volume_file) {
		fprintf(stderr, "APP: astronomy Couldn't find input file %s.\n", VOLUME_FILE);
		exit(-1);
	}

	fscanf(volume_file, "wedge: %d\n", &wedge);
	fscanf(volume_file, "r_steps: %d\n", &r_steps);
	fscanf(volume_file, "mu_steps: %d\n", &mu_steps);
	fscanf(volume_file, "nu_steps: %d\n", &nu_steps);

	if (wedge < 0) {
		nu_min = -90.0;
		nu_max = 90.0;
		mu_min = 0.0;
		mu_max = 360.0;
		r_min = 0;
		r_max = 91.2;
	} else {
		nu_min = -1.25;
		nu_max = 1.25;
		
		for (i = 0; i < num_limits; i++) {
			if (limits[i][0] == wedge) {
				mu_min = limits[i][1];
				mu_max = limits[i][2];
				break;
			}
		}
		
		r_min = (5.0 * log(0.7 * 100.0)/log(10.0)) + 4.2;
		r_max = (5.0 * log(45.7 * 100.0)/log(10.0)) + 4.2;
	}
	r_step = (r_max-r_min)/(double)r_steps;
	mu_step = (mu_max-mu_min)/mu_steps;
	nu_step = (nu_max-nu_min)/nu_steps;

	/*fprintf(stderr,"read volume from: %s\n", VOLUME_FILE);
	fprintf(stderr,"\twedge: %d\n", wedge);
	fprintf(stderr,"\tr min: %lf, max: %lf, steps: %d\n", r_min, r_max, r_steps);
	fprintf(stderr,"\tmu min: %lf, max: %lf, steps: %d\n", mu_min, mu_max, mu_steps);
	fprintf(stderr,"\tnu min: %lf, max: %lf, steps: %d\n", nu_min, nu_max, nu_steps);
	fprintf(stderr,"\n");*/

	r_min_step = ((double)rank/(double)slices) * r_steps;
	r_max_step = (((double)rank+1.0)/(double)slices) * r_steps;


	r_steps = (long)r_max_step - (long)r_min_step;

	r_max = r_min + (((long)r_max_step) * r_step);
	r_min = r_min + (((long)r_min_step) * r_step);

//	printf("process %d read integral data: r_min: %lf, r_max: %lf, r_step: %lf, r_steps: %ld\n", rank, r_min, r_max, r_step, r_steps);
}
