#include <cstdio>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "boinc_genetic_population.h"

void print_statistics(FILE* out_file, POPULATION population) {
	fprintf(out_file, "max: %lf, median: %lf, min: %lf", 
		population.fitness[population.current_population_size-1],
		population.fitness[population.current_population_size/2],
		population.fitness[0]);
}


void read_population(FILE* file, POPULATION *population) {
	int i, j, name_size;

	fscanf(file, "%d: ", &name_size);
	population->name = (char*)malloc(sizeof(char) * name_size);
	fscanf(file, "%s\n",				population->name);

	fscanf(file, "update rate: %d\n",		&population->update_rate);
	population->inserts_since_update = 0;

	fscanf(file, "evaluations done: %d\n",		&population->evaluations_done);
	fscanf(file, "max evaluations allowed: %d\n",	&population->max_evaluations_allowed);

	fscanf(file, "unsent workunit buffer: %d\n",	&population->unsent_workunit_buffer);

	fscanf(file, "current population size: %d\n",	&population->current_population_size);
	fscanf(file, "max population size: %d\n",	&population->max_population_size);

	fscanf(file, "reproduction chance: %lf\n",	&population->reproduction_chance);
	fscanf(file, "double shot: %d\n",		&population->double_shot);

	fscanf(file, "parameter length: %d\n",		&population->parameter_length);

	population->parameter_min = (double*)malloc(sizeof(double) * population->parameter_length);
	population->parameter_max = (double*)malloc(sizeof(double) * population->parameter_length);
	population->fitness = (double*)malloc(sizeof(double) * population->max_population_size);
	population->parameters = (double**)malloc(sizeof(double) * population->max_population_size);
	for (i = 0; i < population->max_population_size; i++) {
		population->parameters[i] = (double*)malloc(sizeof(double) * population->parameter_length);
	}

	fscanf(file, "parameter minimums:");
	for (i = 0; i < population->parameter_length; i++) {
		fscanf(file, " %lf", &population->parameter_min[i]);
	}
	fscanf(file, "\nparameter maximums:");
	for (i = 0; i < population->parameter_length; i++) {
		fscanf(file, " %lf", &population->parameter_max[i]);
	}
	fscanf(file, "\npopulation [fitness : parameters]\n");
	for (i = 0; i < population->current_population_size; i++) {
		fscanf(file, "%lf :", &population->fitness[i]);
		for (j = 0; j < population->parameter_length; j++) {
			fscanf(file, " %lf", &population->parameters[i][j]);
		}
		fscanf(file, "\n");
	}
}

void write_population(FILE* file, POPULATION population) {
	int i, j;
	fprintf(file, "%d: %s\n",			strlen(population.name), population.name);
	fprintf(file, "update rate: %d\n",		population.update_rate);

	fprintf(file, "evaluations done: %d\n",		population.evaluations_done);
	fprintf(file, "max evaluations allowed: %d\n",	population.max_evaluations_allowed);

	fprintf(file, "unsent workunit buffer: %d\n",	population.unsent_workunit_buffer);

	fprintf(file, "current population size: %d\n",	population.current_population_size);
	fprintf(file, "max population size: %d\n",	population.max_population_size);

	fprintf(file, "reproduction chance: %lf\n",	population.reproduction_chance);
	fprintf(file, "double shot: %d\n",		population.double_shot);

	fprintf(file, "parameter length: %d\n",		population.parameter_length);
	fprintf(file, "parameter minimums:");
	for (i = 0; i < population.parameter_length; i++) {
		fprintf(file, " %lf", population.parameter_min[i]);
	}
	fprintf(file, "\nparameter maximums:");
	for (i = 0; i < population.parameter_length; i++) {
		fprintf(file, " %lf", population.parameter_max[i]);
	}
	fprintf(file, "\npopulation [fitness : parameters]\n");
	for (i = 0; i < population.current_population_size; i++) {
		fprintf(file, "%lf :", population.fitness[i]);
		for (j = 0; j < population.parameter_length; j++) {
			fprintf(file, " %lf", population.parameters[i][j]);
		}
		fprintf(file, "\n");
	}
}

double* generate_random_parameters(POPULATION population) {
	int i;
	double* random_parameters;

	random_parameters = (double*)malloc(sizeof(double) * population.parameter_length);

	for (i = 0; i < population.parameter_length; i++) {
		random_parameters[i] = population.parameter_min[i] + (drand48() * (population.parameter_max[i]-population.parameter_min[i]));
	}
	return random_parameters;
}

double* generate_mutation(POPULATION population) {
	int target, target_parameter;
	target = (int)(drand48() * population.current_population_size);
	target_parameter = (int)(drand48() * population.parameter_length);
	double* reproduced_parameters;
	reproduced_parameters = (double*)malloc(sizeof(double) * population.parameter_length);
	memcpy(reproduced_parameters, population.parameters[target], sizeof(double) * population.parameter_length);

	reproduced_parameters[target_parameter] = population.parameter_min[target_parameter] + (drand48() * (population.parameter_max[target_parameter] - population.parameter_min[target_parameter]));
	return reproduced_parameters;
}

double* generate_reproduction(POPULATION population) {
	int i;
	double* reproduced_parameters;
	reproduced_parameters = (double*)malloc(sizeof(double) * population.parameter_length);
	for (i = 0; i < population.parameter_length; i++) {
		reproduced_parameters[i] = (population.parameter_max[i]+population.parameter_min[i])/2;
	}
	return reproduced_parameters;
}

double** generate_double_shot(POPULATION population) {
	int i, first, second;
	double diff;
	double** reproduced_parameters;
	
	first = (int)(drand48() * population.current_population_size);
	second = (int)(drand48() * population.current_population_size);
	while (second == first) second = (int)(drand48() * population.current_population_size);

	reproduced_parameters = (double**)malloc(sizeof(double*) * 3);
	reproduced_parameters[0] = (double*)malloc(sizeof(double) * population.parameter_length);
	reproduced_parameters[1] = (double*)malloc(sizeof(double) * population.parameter_length);
	reproduced_parameters[2] = (double*)malloc(sizeof(double) * population.parameter_length);

	for (i = 0; i < population.parameter_length; i++) {
		reproduced_parameters[0][i] = (population.parameters[first][i]+population.parameters[second][i])/2;
		diff = reproduced_parameters[0][i] - population.parameters[first][i];
		reproduced_parameters[0][i] = population.parameters[first][i] - diff;
		reproduced_parameters[0][i] = population.parameters[second][i] + diff;

	}
	return reproduced_parameters;
}

void insert_member(POPULATION *population, double* parameters, double fitness) {
	if (isnan(fitness)) return;

	if (population->current_population_size == 0) {
		memcpy(population->parameters[0], parameters, sizeof(double) * population->parameter_length);
		population->fitness[0] = fitness;
		population->current_population_size = 1;
		population->inserts_since_update++;
	} else {
		int position = population->current_population_size;
		int i;

		for (i = 0; i < population->current_population_size; i++) {
			if (fitness < population->fitness[i]) {
				position = i;
				break;
			} else if (fitness == population->fitness[i]) {
				return;
			}
		}

		if (population->current_population_size < population->max_population_size) {
			population->current_population_size++;
		}

		if (position < population->current_population_size) {
			for (i = population->current_population_size-1; i > position; i--) {
				memcpy(population->parameters[i], population->parameters[i-1], sizeof(double) * population->parameter_length);
				population->fitness[i] = population->fitness[i-1];
			}
			memcpy(population->parameters[position], parameters, sizeof(double) * population->parameter_length);
			population->fitness[position] = fitness;
			population->inserts_since_update++;
		}
	}
}
