#include <cstdio>
#include <string.h>
#include <stdlib.h>

#include "boinc_genetic_search.h"
#include "boinc_genetic_population.h"

#define results_path "/export/www/boinc/milkyway/results/genetic_search/"
#define log_file "/export/www/boinc/milkyway/results/genetic_search/gs_log"

FILE* log;
int wu_buffer = 30;
int watched_populations = 0;
POPULATION *populations;

void get_population_file(char *search_id, char **population_file) {
	(*population_file) = (char*)malloc(sizeof(results_path) + sizeof(search_id) + sizeof(char)*11);
	memcpy((*population_file), results_path, strlen(results_path));
	(*population_file)[strlen(results_path)] = '\0';
	strcat((*population_file), search_id);
	strcat((*population_file), "/population");
}

void get_population_output(char *search_id, char** population_file) {
	(*population_file) = (char*)malloc(sizeof(results_path) + sizeof(search_id) + sizeof(char)*7);
	memcpy((*population_file), results_path, strlen(results_path));
	(*population_file)[strlen(results_path)] = '\0';
	strcat((*population_file), search_id);
	strcat((*population_file), "/output");
}

POPULATION* get_population(char *search_id) {
	if (watched_populations == 0) {
		watched_populations = 1;
		populations = (POPULATION*)malloc(sizeof(POPULATION));

		char *population_file_name;
		get_population_file(search_id, &population_file_name);

		FILE* population_file = fopen(population_file_name, "r");
		if (population_file == NULL) {
			fprintf(log, "[%s] encounterd error opening population file for read [%d]: %s\n", search_id, population_file, population_file_name);
		} else {
			read_population(population_file, &populations[0]);
			fclose(population_file);
			fprintf(log, "[%s] read population file: %s\n", search_id, population_file_name);
		}
		free(population_file_name);
		return &populations[0];
	} else {
		int i;
		for (i = 0; i < watched_populations; i++) {
			if (strcmp(populations[i].name, search_id) == 0) {
				return &populations[i];
			}
		}

		populations = (POPULATION*)realloc(populations, sizeof(POPULATION) * i);

		char *population_file_name;
		get_population_file(search_id, &population_file_name);
		FILE* population_file = fopen(population_file_name, "r");
		if (population_file == NULL) {
			fprintf(log, "[%s] encounterd error opening population file for read [%d]: %s\n", search_id, population_file, population_file_name);
		} else {
			read_population(population_file, &populations[i-1]);
			fclose(population_file);
			fprintf(log, "[%s] read population file: %s\n", search_id, population_file_name);
		}
		free(population_file_name);
		return &populations[i-1];
	}
}

int gs__process_workunit(char* search_id, double* parameters_used, double fitness) {
	int i;
	/*
	 *	Determine the target population from the work unit name.
	 */
	log = fopen(log_file, "a+");

	POPULATION *target = get_population(search_id);
	fprintf(log, "[%s] determined target: %s\n", search_id, target->name);

	if (target->evaluations_done >= target->max_evaluations_allowed) {
		fprintf(log, "[%s] received work, but search has completed.  discarding.\n", search_id);
	}
	target->evaluations_done++;

	/*
	 *	Insert the results into the population
	 */
	insert_member(target, parameters_used, fitness);
	fprintf(log, "[%s] inserted member.\n", search_id);

	char *output_file;
	get_population_output(search_id, &output_file);
	FILE *output = fopen(output_file, "a+");
	if (output == NULL) {
		fprintf(log, "[%s] encountered error opening population output_file for append [%d]: %s\n", search_id, output, output_file);
	} else {
		fprintf(output, "[%s] updated: %lf, ", search_id, fitness);
		print_statistics(output, *target);
		fprintf(output, ", parameters:");
		for (i = 0; i < target->parameter_length; i++) {
			fprintf(output, " %lf", parameters_used[i]);
		}
		fprintf(output, "\n");
		fclose(output);
	}
	free(output_file);

	/*
	 *	Write the population back to a file if required.
	 */
	if (target->inserts_since_update >= target->update_rate) {
		char *population_file_name;
		get_population_file(search_id, &population_file_name);
		FILE* population_file = fopen(population_file_name, "w+");
		if (population_file == NULL) {
			fprintf(log, "[%s] encounterd error opening population file for write [%d]: %s\n", search_id, population_file, population_file_name);
		} else {
			write_population(population_file, *target);
			fclose(population_file);
			target->inserts_since_update = 0;
			fprintf(log, "[%s] updated population file: %s\n", search_id, population_file_name);
		}
		free(population_file_name);
	}
	fclose(log);
	return 0;
}

int gs__generate_workunits(char* search_id, void (*add_work_unit)(char*, double*)) {
	log = fopen(log_file, "a+");
	POPULATION *target = get_population(search_id);

	if (target->evaluations_done >= target->max_evaluations_allowed) {
		fprintf(log, "[%s] completed search, not generating more work.\n", search_id);
		fclose(log);
		return 0;
	}

	/*
	 *	Generate new work units if necessary
	 *		check to see if unsent workunits < buffer
	 *		generate required new work units
	 */
	fprintf(log, "[%s] generating %d work units\n", search_id, target->unsent_workunit_buffer);
	int i;

	for (i = 0; i < target->unsent_workunit_buffer; i++) {
		if (target->current_population_size < target->max_population_size) {
			add_work_unit(search_id, generate_random_parameters(*target));
		} else {
			if (drand48() >= target->reproduction_chance) {
				add_work_unit(search_id, generate_mutation(*target));
			} else {
				if (target->double_shot > 0) {
					double** params = generate_double_shot(*target);
					add_work_unit(search_id, params[0]);
					add_work_unit(search_id, params[1]);
					add_work_unit(search_id, params[2]);
				} else {
					add_work_unit(search_id, generate_reproduction(*target));
				}
			}
		}
	}
	fflush(log);
	fclose(log);
	return 0;
}
