/*
 *  likelihood_function.c
 *  Astronomy
 *
 *  Created by Travis Desell on 2/21/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "mpi.h"

#include "integral_function.h"
#include "likelihood_function.h"
#include "star_points.h"
#include "volume.h"
#include "parameters.h"
#include "probability.h"

double calculate_likelihood(double background_integral, double *stream_integrals) {
	int wedge;
	double background_weight;
	double *background_parameters;
	double *stream_weights, *exp_stream_weights;
	double **stream_parameters;
	double **star_points;
	double sum_exp_weights, prob_sum, star_prob, sum_integrals;
	double *star_coords;
	int num_zero, bad_jacobians;
	int i,j;
	
	double start, finish;

	bool new_formula, convolve;
	new_formula = false;
	convolve = false;

	start = MPI_Wtime();

	wedge = get_wedge();
	background_weight = get_background_weight();
	background_parameters = get_background_parameters();
	stream_weights = get_stream_weights();
	stream_parameters = get_stream_parameters();
	star_points = get_star_points();

	background_weight = exp(background_weight);
	sum_exp_weights = 0.0;
	exp_stream_weights = (double*)malloc(sizeof(stream_weights));
	for (i = 0; i < get_number_streams(); i++) {
		exp_stream_weights[i] = exp(stream_weights[i]);
		sum_exp_weights += exp(stream_weights[i]);
	}
	//update: added background_weight to sum_exp_weights
	sum_exp_weights += background_weight;

	prob_sum = 0.0;
	num_zero = 0;
	bad_jacobians = 0;   

//	fprintf(stderr, "exp(bg_weight): %lf, exp(st_weight): %lf\n", background_weight, sum_exp_weights);

	for (i = 0; i < get_number_star_points(); i++) {
		star_prob = 0.0;
		star_coords = star_points[i];

		//Find the stream probability
		sum_integrals = 0.0;
		for (j = 0; j < get_number_streams(); j++) {
			if (new_formula) {
				//update: added wedge arg to stPsg call
				if (convolve) ;//star_prob += stPsgConvolved(star_coords, stream_parameters[j], wedge) * stream_weights[i];
				else star_prob += stPsg(star_coords, stream_parameters[j]) * exp_stream_weights[j];
				sum_integrals += exp_stream_weights[j] * stream_integrals[j];
			} else {
				//update: added wedge arg to stPsg call
				if (convolve) ;//star_prob += (stPsgConvolved(star_coords, stream_parameters[j], wedge)/stream_integrals[j]) * stream_weights[i];
				else star_prob += (stPsg(star_coords, stream_parameters[j])/stream_integrals[j]) * exp_stream_weights[j];
			}
		}

		//Find the background probability
		if (new_formula) {
			if (convolve) ;//star_prob += stPbxConvolved(star_coords, background_parameters) * background_weight;
			else star_prob += stPbx(star_coords, background_parameters) * background_weight;
			sum_integrals += background_weight * background_integral;
		} else {
			if (convolve) ;//star_prob += (stPbxConvolved(star_coords, background_parameters)/background_integral) * background_weight;
			else star_prob += (stPbx(star_coords, background_parameters)/background_integral) * background_weight;
		}

//		fprintf(stderr, "star_prob: %lf, sum_exp_weights: %lf, background_weight: %lf\n", star_prob, sum_exp_weights, background_weight);

		//Calculate the probability for this star.
		star_prob /= sum_exp_weights;
		if (new_formula) star_prob /= sum_integrals;		
		
		//update: check star_prob==0, not prob_sum
		if (star_prob != 0.0) {
			prob_sum += log(star_prob)/log(10);
		} else {
			num_zero++;
			prob_sum -= 238.0;
		}

//		if (i%50000 == 0) {
//			fprintf(stderr, "likelihood iteration: %d, star_prob: %lf, prob_sum: %lf\n", i, star_prob, prob_sum);
//		}
//		fprintf(stderr, "likelihood iteration: %d, star_prob: %lf, prob_sum: %lf, star_coords: %lf %lf %lf\n", i, star_prob, prob_sum, star_coords[0], star_coords[1], star_coords[2]);
	}
//	prob_sum /= (get_number_star_points()-bad_jacobians);

//	fprintf(stderr, "likelihood function finished, prob_sum: %lf, star_points: %d, bad_jacobians: %d\n", prob_sum, get_number_star_points(), bad_jacobians);

	finish = MPI_Wtime();
//	fprintf(stdout, "likelihood function finished in %lf, prob_sub: %lf\n", (finish-start), prob_sum);
//	fprintf(stdout, "integral and likelihood calculation took: %lf\n", (finish-start) + get_integral_time());

	return prob_sum;
}


