/*
 *  parameters.c
 *  Astronomy
 *
 *  Created by Travis Desell on 2/21/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include <stdbool.h>
#include <stdlib.h>

/****
	*	Astronomy includes
*****/
#include "parameters.h"


/****
	*	Parameter variables
*****/
#define PARAMETER_FILE "parameters.txt"


int number_background_parameters;
double background_weight;
double *background_parameters, *background_parameter_h, *background_parameter_mutation_range, *background_parameter_min, *background_parameter_max;
bool *background_parameter_optimize;

int number_streams;
int number_stream_parameters;
double *stream_weights, *stream_weight_h, *stream_weight_mutation_range, *stream_weight_min, *stream_weight_max;
bool *stream_weight_optimize;

double **stream_parameters, **stream_parameter_h, **stream_parameter_mutation_range, **stream_parameter_min, **stream_parameter_max;
bool **stream_parameter_optimize;

/********
	*	Functions for accessing parameters.
*********/
int		get_number_streams()			{ return number_streams; }
int		get_number_stream_parameters()		{ return number_stream_parameters; }

double		get_background_weight()			{ return background_weight; }
double*		get_background_parameters()		{ return background_parameters; }
double*		get_stream_weights()			{ return stream_weights; }
double**	get_stream_parameters()			{ return stream_parameters; }

/********
	*	Functions for conversion for use by MPI.
 ********/

/********
	*	Functions for use by GML searches.
 ********/

void get_random_astro_parameters(double** parameters, int *parameter_length) {
	int length, current;
	int i, j;
	length = 0;

	for (i = 0; i < number_background_parameters; i++) {
		if (background_parameter_optimize[i]) length++;
	}

	for (i = 0; i < number_streams; i++) {
		if (stream_weight_optimize[i]) length++;

		for (j = 0; j < number_stream_parameters; j++) {
			if (stream_parameter_optimize[i][j]) length++;
		}
	}

	*parameters = (double*)malloc(sizeof(double) * length);
	*parameter_length = length;

	current = 0;
	for (i = 0; i < number_background_parameters; i++) {
		if (background_parameter_optimize[i]) {
			(*parameters)[current] = (drand48() * (background_parameter_max[i]-background_parameter_min[i])) + background_parameter_min[i];
			current++;
		}
	}

	for (i = 0; i < number_streams; i++) {
		if (stream_weight_optimize[i]) {
			(*parameters)[current] = (drand48() * (stream_weight_max[i]-stream_weight_min[i])) + stream_weight_min[i];
			current++;
		}

		for (j = 0; j < number_stream_parameters; j++) {
			if (stream_parameter_optimize[i][j]) {
				(*parameters)[current] = (drand48() * (stream_parameter_max[i][j]-stream_parameter_min[i][j])) + stream_parameter_min[i][j];
				current++;
			}
		}
	}
}

void get_parameters_as_double_array(double** parameters, int *parameter_length) {
	int length, current;
	int i, j;
	length = 0;

	for (i = 0; i < number_background_parameters; i++) {
		if (background_parameter_optimize[i]) length++;
	}

	for (i = 0; i < number_streams; i++) {
		if (stream_weight_optimize[i]) length++;

		for (j = 0; j < number_stream_parameters; j++) {
			if (stream_parameter_optimize[i][j]) length++;
		}
	}

	*parameters = (double*)malloc(sizeof(double) * length);
	*parameter_length = length;

	current = 0;
	for (i = 0; i < number_background_parameters; i++) {
		if (background_parameter_optimize[i]) {
			(*parameters)[current] = background_parameters[i];
			current++;
		}
	}

	for (i = 0; i < number_streams; i++) {
		if (stream_weight_optimize[i]) {
			(*parameters)[current] = stream_weights[i];
			current++;
		}

		for (j = 0; j < number_stream_parameters; j++) {
			if (stream_parameter_optimize[i][j]) {
				(*parameters)[current] = stream_parameters[i][j];
				current++;
			}
		}
	}
}

void set_parameters_from_double_array(double* parameters) {
	int i, j;
	int current;
	current = 0;
	for (i = 0; i < number_background_parameters; i++) {
		if (background_parameter_optimize[i]) {
			background_parameters[i] = parameters[current];
			current++;
		}
	}

	for (i = 0; i < number_streams; i++) {
		if (stream_weight_optimize[i]) {
			stream_weights[i] = parameters[current];
			current++;
		}

		for (j = 0; j < number_stream_parameters; j++) {
			if (stream_parameter_optimize[i][j]) {
				stream_parameters[i][j] = parameters[current];
				current++;
			}
		}
	}
}

void get_step_sizes_as_double_array(double** parameters, int *parameter_length) {
	int length, current;
	int i, j;
	length = 0;

	for (i = 0; i < number_background_parameters; i++) {
		if (background_parameter_optimize[i]) length++;
	}

	for (i = 0; i < number_streams; i++) {
		if (stream_weight_optimize[i]) length++;

		for (j = 0; j < number_stream_parameters; j++) {
			if (stream_parameter_optimize[i][j]) length++;
		}
	}

	*parameters = (double*)malloc(sizeof(double) * length);
	*parameter_length = length;

	current = 0;
	for (i = 0; i < number_background_parameters; i++) {
		if (background_parameter_optimize[i]) {
			(*parameters)[current] = background_parameter_h[i];
			current++;
		}
	}

	for (i = 0; i < number_streams; i++) {
		if (stream_weight_optimize[i]) {
			(*parameters)[current] = stream_weight_h[i];
			current++;
		}

		for (j = 0; j < number_stream_parameters; j++) {
			if (stream_parameter_optimize[i][j]) {
				(*parameters)[current] = stream_parameter_h[i][j];
				current++;
			}
		}
	}
}

double* get_parameter_min() {
	int length, current;
	int i, j;
	double* parameter_min;
	length = 0;

	for (i = 0; i < number_background_parameters; i++) {
		if (background_parameter_optimize[i]) length++;
	}

	for (i = 0; i < number_streams; i++) {
		if (stream_weight_optimize[i]) length++;

		for (j = 0; j < number_stream_parameters; j++) {
			if (stream_parameter_optimize[i][j]) length++;
		}
	}

	parameter_min = (double*)malloc(sizeof(double) * length);

	current = 0;
	for (i = 0; i < number_background_parameters; i++) {
		if (background_parameter_optimize[i]) {
			parameter_min[current] = background_parameter_min[i];
			current++;
		}
	}

	for (i = 0; i < number_streams; i++) {
		if (stream_weight_optimize[i]) {
			parameter_min[current] = stream_weight_min[i];
			current++;
		}

		for (j = 0; j < number_stream_parameters; j++) {
			if (stream_parameter_optimize[i][j]) {
				parameter_min[current] = stream_parameter_min[i][j];
				current++;
			}
		}
	}
	return parameter_min;
}

double* get_parameter_max() {
	int length, current;
	int i, j;
	double* parameter_max;
	length = 0;

	for (i = 0; i < number_background_parameters; i++) {
		if (background_parameter_optimize[i]) length++;
	}

	for (i = 0; i < number_streams; i++) {
		if (stream_weight_optimize[i]) length++;

		for (j = 0; j < number_stream_parameters; j++) {
			if (stream_parameter_optimize[i][j]) length++;
		}
	}

	parameter_max = (double*)malloc(sizeof(double) * length);

	current = 0;
	for (i = 0; i < number_background_parameters; i++) {
		if (background_parameter_optimize[i]) {
			parameter_max[current] = background_parameter_max[i];
			current++;
		}
	}

	for (i = 0; i < number_streams; i++) {
		if (stream_weight_optimize[i]) {
			parameter_max[current] = stream_weight_max[i];
			current++;
		}

		for (j = 0; j < number_stream_parameters; j++) {
			if (stream_parameter_optimize[i][j]) {
				parameter_max[current] = stream_parameter_max[i][j];
				current++;
			}
		}
	}
	return parameter_max;
}



/****
	*	Functions for printing parameters to files/output
*****/
void print_double_array(FILE *file, char *array_name, int size, double *array) {
	int i;
	
	fprintf(file, array_name);
	for (i = 0; i < size; i++) {
		fprintf(file, "%lf", array[i]);
		if (i < size-1) fprintf(file, ", ");
	}
	fprintf(file, "\n");
}

void print_boolean(FILE *file, bool value) {
	if (value) {
		fprintf(file, "true");
	} else {
		fprintf(file, "false");
	}
}

void print_boolean_array(FILE *file, char *array_name, int size, bool *array) {
	int i;
	
	fprintf(file, array_name);
	for (i = 0; i < size; i++) {
		print_boolean(file, array[i]);
		if (i < size-1) fprintf(file, ", ");
	}
	fprintf(file, "\n");
}

void print_parameters(FILE *file) {
	int i;
	
	fprintf(file, "number_parameters: %d\n", number_background_parameters);
	fprintf(file, "background_weight: %lf\n", background_weight);
	print_double_array(file, "background_parameters: ", number_background_parameters, background_parameters);
	print_double_array(file, "background_h: ", number_background_parameters, background_parameter_h);
	print_double_array(file, "background_mutation_range: ", number_background_parameters, background_parameter_mutation_range);
	print_double_array(file, "background_parameter_min: ", number_background_parameters, background_parameter_min);
	print_double_array(file, "background_parameter_max: ", number_background_parameters, background_parameter_max);
	print_boolean_array(file, "optimize_parameter: ", number_background_parameters, background_parameter_optimize);
	
	fprintf(file, "number_streams: %d, %d\n", number_streams, number_stream_parameters);
	for (i = 0; i < number_streams; i++) {
		fprintf(file, "stream_weight: %lf\n", stream_weights[i]);
		fprintf(file, "stream_weight_h: %lf\n", stream_weight_h[i]);
		fprintf(file, "stream_weight_mutation_range: %lf\n", stream_weight_mutation_range[i]);
		fprintf(file, "stream_weight_min: %lf\n", stream_weight_min[i]);
		fprintf(file, "stream_weight_max: %lf\n", stream_weight_max[i]);
		fprintf(file, "optimize_weight: ");
		print_boolean(file, stream_weight_optimize[i]);
		fprintf(file, "\n");

		print_double_array(file, "stream_parameters: ", number_stream_parameters, stream_parameters[i]);
		print_double_array(file, "stream_h: ", number_stream_parameters, stream_parameter_h[i]);
		print_double_array(file, "stream_mutation_range: ", number_stream_parameters, stream_parameter_mutation_range[i]);
		print_double_array(file, "stream_parameter_min: ", number_stream_parameters, stream_parameter_min[i]);
		print_double_array(file, "stream_parameter_max: ", number_stream_parameters, stream_parameter_max[i]);
		print_boolean_array(file, "optimize_parameter: ", number_stream_parameters, stream_parameter_optimize[i]);
	}
	fprintf(file, "\n");
}


/****
	*	Functions for reading parameters from files
*****/
void read_double_array(FILE *file, char *array_name, int size, double **array) {
	int i;

	*array = (double*)malloc(sizeof(double)*size);
	fscanf(file, array_name);
	for (i = 0; i < size; i++) {
		fscanf(file, "%lf", &(*array)[i]);
		if (i < size-1) fscanf(file, ", ");
	}
	fscanf(file, "\n");
}

void read_boolean(FILE *file, bool *value) {
	char c;
	c = fgetc(file);
	fgetc(file);
	fgetc(file);
	fgetc(file);
	if (c == 'f') {
		fgetc(file);
		*value = false;
	} else {
		*value = true;
	}
}

void read_boolean_array(FILE *file, char *array_name, int size, bool **array) {
	int i;
	
	*array = (bool*)malloc(sizeof(bool)*size);
	fscanf(file, array_name);
	for (i = 0; i < size; i++) {
		read_boolean(file, &(*array)[i]);
		if (i < size-1) fscanf(file, ", ");
	}
	fscanf(file, "\n");
}

void read_parameters() {
	char file_path[512];
	FILE *parameter_file;
	int i;
	
//	fprintf(stderr, "reading parameters file: %s\n", PARAMETER_FILE);
	parameter_file = fopen(PARAMETER_FILE, "r");
	if (!parameter_file) {
		fprintf(stderr, "Couldn't find input file, resolved name %s.\n", file_path);
		exit(-1);
	}
//	fprintf(stderr, "opened file, resolved path: %s\n", PARAMETER_FILE, file_path);

	fscanf(parameter_file, "number_parameters: %d\n", &number_background_parameters);
	fscanf(parameter_file, "background_weight: %lf\n", &background_weight);
	
	read_double_array(parameter_file, "background_parameters: ", number_background_parameters, &background_parameters);
	read_double_array(parameter_file, "background_h: ", number_background_parameters, &background_parameter_h);
	read_double_array(parameter_file, "background_mutation_range: ", number_background_parameters, &background_parameter_mutation_range);
	read_double_array(parameter_file, "background_parameter_min: ", number_background_parameters, &background_parameter_min);
	read_double_array(parameter_file, "background_parameter_max: ", number_background_parameters, &background_parameter_max);
	read_boolean_array(parameter_file, "optimize_parameter: ", number_background_parameters, &background_parameter_optimize);

	fscanf(parameter_file, "number_streams: %d, %d\n", &number_streams, &number_stream_parameters);
	stream_weights					= (double*)malloc(sizeof(double) * number_streams);
	stream_weight_h					= (double*)malloc(sizeof(double) * number_streams);
	stream_weight_mutation_range			= (double*)malloc(sizeof(double) * number_streams);
	stream_weight_min				= (double*)malloc(sizeof(double) * number_streams);
	stream_weight_max				= (double*)malloc(sizeof(double) * number_streams);
	stream_weight_optimize				=  (bool*)malloc(sizeof(bool)  * number_streams);

	stream_parameters				= (double**)malloc(sizeof(double*) * number_streams);
	stream_parameter_h				= (double**)malloc(sizeof(double*) * number_streams);
	stream_parameter_mutation_range			= (double**)malloc(sizeof(double*) * number_streams);
	stream_parameter_min				= (double**)malloc(sizeof(double*) * number_streams);
	stream_parameter_max				= (double**)malloc(sizeof(double*) * number_streams);
	stream_parameter_optimize			=  (bool**)malloc(sizeof(bool*)   * number_streams);

	for (i = 0; i < number_streams; i++) {
		fscanf(parameter_file, "stream_weight: %lf\n", &stream_weights[i]);
		fscanf(parameter_file, "stream_weight_h: %lf\n", &stream_weight_h[i]);
		fscanf(parameter_file, "stream_weight_mutation_range: %lf\n", &stream_weight_mutation_range[i]);
		fscanf(parameter_file, "stream_weight_min: %lf\n", &stream_weight_min[i]);
		fscanf(parameter_file, "stream_weight_max: %lf\n", &stream_weight_max[i]);
		fscanf(parameter_file, "optimize_weight: ");
		read_boolean(parameter_file, &stream_weight_optimize[i]);
		fscanf(parameter_file, "\n");

		read_double_array(parameter_file, "stream_parameters: ", number_stream_parameters, &stream_parameters[i]);
		read_double_array(parameter_file, "stream_h: ", number_stream_parameters, &stream_parameter_h[i]);
		read_double_array(parameter_file, "stream_mutation_range: ", number_stream_parameters, &stream_parameter_mutation_range[i]);
		read_double_array(parameter_file, "stream_parameter_min: ", number_stream_parameters, &stream_parameter_min[i]);
		read_double_array(parameter_file, "stream_parameter_max: ", number_stream_parameters, &stream_parameter_max[i]);
		read_boolean_array(parameter_file, "optimize_parameter: ", number_stream_parameters, &stream_parameter_optimize[i]);
	}
}
