/*
 *  star_points.c
 *  Astronomy
 *
 *  Created by Travis Desell on 2/21/07.
 *  Copyright 2007 __MyCompanyName__. All rights reserved.
 *
 */

#include "../framework/gml.h"

/****
	*	Astronomy includes
*****/
#include "star_points.h"


/****
	*	Star point variables
*****/
#define STAR_FILE "stars.txt"
long number_star_points;
long total_star_points;
double **star_points;

long		get_total_star_points()		{ return total_star_points; }
long		get_number_star_points()	{ return number_star_points; }
double**	get_star_points()		{ return star_points; }

void read_likelihood_data(int rank, int slices) {
	long slice_min, slice_max, current_slice;
	int i, j;
	double slice_size;
	double** temp_data;

	read_star_points();

	slice_size = (double)number_star_points / (double)slices;
	slice_min = 0;
	for (i = 0; i < slices; i++) {
		if (i == slices-1) {
			slice_max = number_star_points;
		} else {
			slice_max = (i+1) * (slice_size);
		}

		if (i == rank) {
			current_slice = slice_max-slice_min;

			temp_data = (double**)malloc(sizeof(double) * current_slice);
			for (j = 0; j < current_slice; j++) {
				temp_data[j] = (double*)malloc(sizeof(double) * 3);
				temp_data[j][0] = star_points[slice_min + j][0];
				temp_data[j][1] = star_points[slice_min + j][1];
				temp_data[j][2] = star_points[slice_min + j][2];
			}
//			fprintf(gml_get_output(), "splitting likelihood data, min: %ld, max: %ld, size: %ld\n", slice_min, slice_max, current_slice);
			star_points = temp_data;
			number_star_points = current_slice;
			return;
		}
		slice_min = slice_max;
	}
}

void read_star_points() {
	char file_path[512];
	FILE *data_file;
	int i;
	
	data_file = fopen(STAR_FILE, "r");
	if (!data_file) {
		fprintf(stderr, "Couldn't find input file, resolved name %s.\n", file_path);
		exit(-1);
	}
//	fprintf(stderr, "opened file, resolved path: %s\n", STAR_FILE, file_path);

	fscanf(data_file, "%d\n", &number_star_points);
//	fprintf(stderr, "reading %d star points.\n", number_star_points);

	total_star_points = number_star_points;
	star_points = (double**)malloc(sizeof(double*)*number_star_points);
	for (i = 0; i < number_star_points; i++) {
		star_points[i] = (double*)malloc(sizeof(double)*3);
		fscanf(data_file, "%lf %lf %lf\n", &star_points[i][0], &star_points[i][1], &star_points[i][2]);
//		if (i%10000 == 0) fprintf(stderr, "read %d points. last: %lf %lf %lf\n", i, star_points[i][0], star_points[i][1], star_points[i][2]);
	}

//	fprintf(stderr, "finished reading %d star points.\n", number_star_points);
//	fprintf(stderr, "completed read_star_data.\n\n");
}
