/*
 *  genetic_search.h
 *	This is the API for genetic search.
 */
 
#ifndef GENETIC_SEARCH_H
#define GENETIC_SEARCH_H

/*
 *	The following sets the maximum number of iterations to perform.
 */
void genetic_search__max_iterations(int max_iterations);

/*
 *	The following will start a genetic search of the specified likelihood function
 *	within the parameter bounds pasesd to this function.
 *	Iterative genetic search creates a population, then from that population iteratively creates
 *	a new population of the same size and evaluates it.
 *	Probabilistic genetic search creates an initial population, then probabilistically creates new
 *	members of that population, and replaces members having lower likelihood with members having
 *	higher likelihood.
 */
double* genetic_search_iterative(double* parameter_minimums, double* parameter_maximums, int parameter_length);
double* genetic_search_probabilistic(double* parameter_minimums, double* parameter_maximums, int parameter_length, int max_population_size, double reproduce_chance, int number_iterations);
#endif
