#include <stdio.h>
#include <math.h>
#include <string.h>
#include "matrix.h"

MATRIX create_matrix(int x, int y) {
	MATRIX new;
	int i, j;

	new.x = x;
	new.y = y;
	new.values = (double**)malloc(sizeof(double*) * x);
	for (i = 0; i < x; i++) {
		new.values[i] = (double*)malloc(sizeof(double) * y);
		for (j = 0; j < y; j++) {
			new.values[i][j] = 0;
		}
	}

	return new;
}

MATRIX matrix_mul(MATRIX m1, MATRIX m2) {
	MATRIX result;
	int i, j, k;
	if (m1.y == m2.x) {
		result = create_matrix(m1.x, m2.y);
		for (i = 0; i < m1.x; i++) {
			for (j = 0; j < m2.y; j++) {
				for (k = 0; k < m1.y; k++) {
					result.values[i][j] += m1.values[i][k]*m2.values[k][j];
				}
			}
		}
	} else if (m1.x == 1 && m1.y == 1) {
		result = create_matrix(m2.x, m2.y);
		for (i = 0; i < m2.x; i++) {
			for (j = 0; j < m2.y; j++) {
				result.values[i][j] = m2.values[i][j] * m1.values[0][0];
			}
		}
	} else if (m2.x == 1 && m2.y == 1) {
		result = create_matrix(m1.x, m1.y);
		for (i = 0; i < m1.x; i++) {
			for (j = 0; j < m1.y; j++) {
				result.values[i][j] = m1.values[i][j] * m2.values[0][0];
			}
		}		
	} else {
		fprintf(stderr, "ERROR in matrix multiply, m1.y != m2.x\n");
		fprintf(stderr, "m1.x: %d, m1.y: %d\n", m1.x, m1.y);
		fprintf(stderr, "m2.x: %d, m2.y: %d\n", m2.x, m2.y);
		exit(0);
	}

	return result;
}

MATRIX matrix_add(MATRIX m1, MATRIX m2) {
	MATRIX result;
	int i, j;

	if (m1.x == m2.x && m1.y == m2.y) {
		result = create_matrix(m1.x, m1.y);
		for (i = 0; i < m1.x; i++) {
			for (j = 0; j < m1.y; j++) {
				result.values[i][j] = m1.values[i][j] + m2.values[i][j];
			}
		}
	} else {
		fprintf(stderr, "ERROR in matrix add, dimensions do not match!");
		exit(0);
	}

	return result;
}

MATRIX matrix_sub(MATRIX m1, MATRIX m2) {
	MATRIX result;
	int i, j;

	if (m1.x == m2.x && m1.y == m2.y) {
		result = create_matrix(m1.x, m1.y);
		for (i = 0; i < m1.x; i++) {
			for (j = 0; j < m1.y; j++) {
				result.values[i][j] = m1.values[i][j] - m2.values[i][j];
			}
		}
	} else {
		fprintf(stderr, "ERROR in matrix sub, dimensions do not match!");
		exit(0);
	}

	return result;
}

MATRIX invert(MATRIX m) {
	MATRIX result;
	int i, j;
	result = create_matrix(m.y, m.x);

	for (i = 0; i < m.x; i++) {
		for (j = 0; j < m.y; j++) {
			result.values[j][i] = m.values[i][j];
		}
	}
	return result;
}

MATRIX matrix_rdiv(MATRIX m1, MATRIX m2) {
	int i, j;

	if (m2.x == 1 && m2.y == 1) {
		MATRIX result = create_matrix(m1.x, m1.y);
		for (i = 0; i < m1.x; i++) {
			for (j = 0; j < m1.y; j++) {
				if (m1.values[i][j] == 0 && m2.values[0][0] == 0) {
					result.values[i][j] = 0;
				} else {
					result.values[i][j] = m1.values[i][j]/m2.values[0][0];
				}
			}
		}
		return result;
	} else if (m2.x == m2.y) {
		return matrix_mul(m1, invert(m2));
	} else {
		fprintf(stderr, "matrix_rdiv error: m2 not square\n");
		exit(0);
	}
}

double* matrix2array(MATRIX m) {
	int i;
	if (m.y != 1) {
		fprintf(stderr, "ERROR matrix2array won't work because matrix.y != 1");
		exit(0);
	}

	double* result = (double*)malloc(sizeof(double) * m.x);
	for (i = 0; i < m.x; i++) {
		result[i] = m.values[i][0];
	}
	return result;
}

double mean(MATRIX m) {
	int i, j;
	int count = 0;
	double total = 0.0;
	for (i = 0; i < m.x; i++) {
		for (j = 0; j < m.x; j++) {
			total += m.values[i][j];
			count++;
		}
	}
	return total/count;
}

MATRIX matrix_neg(MATRIX m) {
	MATRIX result;
	int i, j;
	result = create_matrix(m.x, m.y);
	for (i = 0; i < m.x; i++) {
		for (j = 0; j < m.y; j++) {
			result.values[i][j] = -m.values[i][j];
		}
	}
	return result;
}

double matrix_norm(MATRIX m) {
	int i, j;
	double result = 0.0;
	for (i = 0; i < m.x; i++) {
		for (j = 0; j < m.y; j++) {
			result += sqrt(m.values[i][j]*m.values[i][j]);
		}
	}
	return result;
}

MATRIX matrix_apply_mul(MATRIX m, double d) {
	int i, j;
	MATRIX result = create_matrix(m.x, m.y);

	for (i = 0; i < m.x; i++) {
		for (j = 0; j < m.y; j++) {
			result.values[i][j] = m.values[i][j] * d;
		}
	}
	return result;
}

MATRIX matrix_apply_div(MATRIX m, double d) {
	int i, j;
	MATRIX result = create_matrix(m.x, m.y);
	for (i = 0; i < m.x; i++) {
		for (j = 0; j < m.y; j++) {
			result.values[i][j] = m.values[i][j] / d;
		}
	}
	return result;
}

MATRIX matrix_apply_sub(MATRIX m, double d) {
	int i, j;
	MATRIX result = create_matrix(m.x, m.y);
	for (i = 0; i < m.x; i++) {
		for (j = 0; j < m.y; j++) {
			result.values[i][j] = m.values[i][j] - d;
		}
	}
	return result;
}

void print_matrix(MATRIX m) {
	int i, j;
	for (i = 0; i < m.y; i++) {
		for (j = 0; j < m.x; j++) {
			fprintf(stderr, " %lf", m.values[j][i]);
		}
		fprintf(stderr, "\n");
	}
}
