#include <stdio.h>
#include <time.h>
#include <math.h>
#include <string.h>

#include "evaluator.h"
#include "simplex.h"

double ALPHA = -1.0;
double BETA = 0.5;
double GAMMA = 2.0;

double ftol = 10e-8;	//cutoff

int current_iteration = 0;
int maximum_iterations = 10000;

double* y;
double* psum;
double* ptry;
double** p;
int highest, next_highest, lowest;
double ytry, ysave;

int isGrow = 0;
int isContract = 0;
int isReflect = 0;
int isAugment = 0;
double factor;
double rtol, ptol;

int number_parameters;
int parameter_length;


void simplex__max_iterations(int mi) {
	maximum_iterations = mi;
}

void recalculate() {
	int i;
	highest = 0;
	lowest = 0;
	for (i = 1; i < number_parameters; i++) {
		if (y[i] < y[lowest]) lowest = i;
		if (y[i] > y[highest]) {
			next_highest = highest;
			highest = i;
		}
	}
	next_highest = highest;
	int set_lower = 0;
	for (i = 0; i < number_parameters; i++) {
		if (!set_lower && y[i] != y[highest]) {
			next_highest = i;
			set_lower = 1;
		} else if (set_lower > 0&& y[i] > y[next_highest] && y[i] != y[highest]) {
			next_highest = i;
		}
	}

	rtol = 2.0*fabs(y[highest] - y[lowest])/(fabs(y[highest]) + fabs(y[lowest]));
	ptol = 0.0;
	for (i = 0; i < parameter_length; i++) {
		ptol += fabs(p[highest][i] - p[lowest][i]);
	}
	ptol /= parameter_length;
}

double* step() {
	int i, j;

	recalculate();

	if (ptry != NULL) free(ptry);
	ptry = (double*)malloc(sizeof(double) * parameter_length);
	while (current_iteration < maximum_iterations && (rtol > ftol || ptol > ftol)) {
		fprintf(stderr, "STEPPING. highest[%d]: %lf, lowest[%d]: %lf, next_highest[%d]: %lf\n", highest, y[highest], lowest, y[lowest], next_highest, y[next_highest]);

		isReflect = 1;
		isAugment = 1;
		factor = ALPHA;
		while (isAugment > 0) {
			fprintf(stderr, "Augmenting by: %lf\n", factor);
			isAugment = 0;

			double factor1 = (1.0 - factor)/parameter_length;
			double factor2 = factor1 - factor;

			for (i = 0; i < parameter_length; i++) {
				ptry[i] = psum[i] * factor1 - p[highest][i] * factor2;
			}

			ytry = track_evaluate(ptry);
			fprintf(stderr, "ytry: %lf\n", ytry);

			if (ytry < y[highest]) {
				y[highest] = ytry;
				for (j = 0; j < parameter_length; j++) {
					psum[j] += ptry[j] - p[highest][j];
					p[highest][j] = ptry[j];
				}
			}

			if (isReflect > 0) {
				isReflect = 0;
				fprintf(stderr, "REFLECTING: ytry: %lf, ysave: %lf, lowest y[%d]: %lf, next_highest y[%d]: %lf\n", ytry, ysave, lowest, y[lowest], next_highest, y[next_highest]);

				if (ytry <= y[lowest]) {
					isGrow = 1;
					isAugment = 1;
					factor = GAMMA;
				} else if (ytry >= y[next_highest]) {
					isContract = 1;
					ysave = y[highest];
					isAugment = 1;
					factor = BETA;
				}
			} else if (isGrow > 0) {
				isGrow = 0;
			} else if (isContract > 0) {
				if (ytry >= ysave) {
					isContract = 0;
					fprintf(stderr, "Contracting.\n");

					int current = 0;
					for (i = 0; i < number_parameters; i++) {
						if (i != lowest) {
							for (j = 0; j < parameter_length; j++) {
								psum[j] = 0.5 * (p[i][j] + p[lowest][j]);
								p[i][j] = psum[j];
							}
							current++;
						}
					}
					for (i = 0; i < number_parameters; i++) {
						y[i] = track_evaluate(p[i]);
					}
				} else {
					fprintf(stderr, "CONTRACT DID NOTHING.\n");
				}
			}
		}
		recalculate();
		current_iteration++;
	}
	fprintf(stderr, "current iteration: %d, maximum_iterations: %d\n", current_iteration, maximum_iterations);
	fprintf(stderr, "rtol < ftol? %lf < %lf\n", rtol, ftol);
	fprintf(stderr, "ptol < ftol? %lf < %lf\n", ptol, ftol);
	return p[lowest];
}

double* simplex(double** initial_parameters, int n_p, int p_l) {
	int i, j;

	reset_evaluator();

	current_iteration = 0;
	p = initial_parameters;
	parameter_length = p_l;
	number_parameters = n_p;

	if (psum != NULL) free(psum);
	psum = (double*)malloc(sizeof(double) * parameter_length);
	double sum;
	for (j = 0; j < parameter_length; j++) {
		for (i = 0, sum = 0.0; i < number_parameters; i++) {
			sum += p[i][j];
		}
		psum[j] = sum;
	}

	if (y != NULL) free(y);
	y = (double*)malloc(sizeof(double) * number_parameters);
	for (i = 0; i < number_parameters; i++) {
		y[i] = track_evaluate(p[i]);
	}

	return step();
}
