package gmle.salsa.astronomy;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.FileReader;
import java.io.FileOutputStream;
import java.io.OutputStream;

import java.util.Vector;
import java.util.StringTokenizer;

public class IntegralParameters {
	int convolve;

        double background_weight;
        double[] background_parameters;

	//this is for gradient descent
	double[] background_h;

	//these are for the genetic search
	double[] background_mutation_range;
	double[] background_parameter_min;
	double[] background_parameter_max;
	boolean[] background_optimize_parameter;

        double[] stream_weights;
	double[] stream_weight_h;
	double[] stream_weight_mutation_range;
	double[] stream_weight_min;
	double[] stream_weight_max;
	boolean[] stream_weight_optimize;
	double[][] stream_parameters;

	//this is for gradient descent
	double[][] stream_h;

	//this is for genetic search
	double[][] stream_mutation_range;
	double[][] stream_parameter_min;
	double[][] stream_parameter_max;
	boolean[][] stream_optimize_parameter;

	public int		convolve()			{ return convolve; }

        public	double		getBackgroundWeight()		{ return background_weight; }
        public	double[]	getBackgroundParameters()	{ return background_parameters; }
        public	double[]	getStreamWeights()		{ return stream_weights; }
        public	double[][]	getStreamParameters()		{ return stream_parameters; }
	
	public IntegralParameters() {}

	public IntegralParameters(String file) {
		this.readParametersFromFile(file);
	}

	public IntegralParameters copy() {
		return new IntegralParameters(this);
	}

	public IntegralParameters(IntegralParameters original) {
		this.background_weight = original.background_weight;
		this.convolve = original.convolve;

		this.background_parameters = copyDoubleArray(original.background_parameters);
		this.background_h = copyDoubleArray(original.background_h);
		this.background_mutation_range = copyDoubleArray(original.background_mutation_range);
		this.background_parameter_min = copyDoubleArray(original.background_parameter_min);
		this.background_parameter_max = copyDoubleArray(original.background_parameter_max);
		this.background_optimize_parameter = copyBooleanArray(original.background_optimize_parameter);

		this.stream_weights = copyDoubleArray(original.stream_weights);
		this.stream_weight_h = copyDoubleArray(original.stream_weight_h);
		this.stream_weight_mutation_range = copyDoubleArray(original.stream_weight_mutation_range);
		this.stream_weight_min = copyDoubleArray(original.stream_weight_min);
		this.stream_weight_max = copyDoubleArray(original.stream_weight_max);
		this.stream_weight_optimize = copyBooleanArray(original.stream_weight_optimize);

		this.stream_parameters = copyDouble2Array(original.stream_parameters);
		this.stream_h = copyDouble2Array(original.stream_h);
		this.stream_mutation_range = copyDouble2Array(original.stream_mutation_range);
		this.stream_parameter_min = copyDouble2Array(original.stream_parameter_min);
		this.stream_parameter_max = copyDouble2Array(original.stream_parameter_max);
		this.stream_optimize_parameter = copyBoolean2Array(original.stream_optimize_parameter);
	}

	public double[] copyDoubleArray(double[] array) {
		double[] target = new double[array.length];
		System.arraycopy(array, 0, target, 0, array.length);
		return target;
	}

	public double[][] copyDouble2Array(double[][] array) {
		double[][] target = new double[array.length][];
		for (int i = 0; i < array.length; i++) {
			target[i] = new double[array[i].length];
			System.arraycopy(array[i], 0, target[i], 0, array[i].length);
		}
		return target;
	}

	public boolean[] copyBooleanArray(boolean[] array) {
		boolean[] target = new boolean[array.length];
		System.arraycopy(array, 0, target, 0, array.length);
		return target;
	}

	public boolean[][] copyBoolean2Array(boolean[][] array) {
		boolean[][] target = new boolean[array.length][];
		for (int i = 0; i < array.length; i++) {
			target[i] = new boolean[array[i].length];
			System.arraycopy(array[i], 0, target[i], 0, array[i].length);
		}
		return target;
	}

	public void set_parameters(double[] parameters) {
		this.convertToUnconstrained();
		int current = 0;
		for (int i = 0; i < background_parameters.length; i++) {
			if (background_optimize_parameter[i]) {
				background_parameters[i] = parameters[current];
				current++;
			}
		}

		for (int i = 0; i < stream_parameters.length; i++) {
			if (stream_weight_optimize[i]) {
				stream_weights[i] = parameters[current];
				current++;
			}

			for (int j = 0; j < stream_parameters[i].length; j++) {
				if (stream_optimize_parameter[i][j]) {
					stream_parameters[i][j] = parameters[current];
					current++;
				}
			}
		}
		this.convertToConstrained();
	}

	public double[][] get_simplex_parameters() {
		double[] parameter_min = this.get_parameter_min();
		double[] parameter_max = this.get_parameter_max();

		double[][] simplex_parameters = new double[parameter_min.length+1][parameter_min.length];
		for (int i = 0; i < simplex_parameters.length; i++) {
			for (int j = 0; j < simplex_parameters.length; j++) {
				simplex_parameters[i][j] = parameter_min[j] + ((parameter_max[j]-parameter_min[j])*Math.random());
			}
		}
		return simplex_parameters;
	}

        public double[] get_parameters() {
                this.convertToUnconstrained();

                /********
                        *       Calculate the number of parameters that will
                        *       be optimized over
                 ********/
                int p_count = 0;
                for (int i = 0; i < background_optimize_parameter.length; i++) {
                        if (background_optimize_parameter[i]) p_count++;
                }

                for (int i = 0; i < stream_optimize_parameter.length; i++) {
                        if (stream_weight_optimize[i]) p_count++;
                        for (int j = 0; j < stream_optimize_parameter[i].length; j++) {
                                if (stream_optimize_parameter[i][j]) p_count++;
                        }
                }

                double[] parameters = new double[p_count];
                int current = 0;
                for (int i = 0; i < background_parameters.length; i++) {
                        if (background_optimize_parameter[i]) {
                                parameters[current] = background_parameters[i];
                                current++;
                        }
                }

                for (int i = 0; i < stream_parameters.length; i++) {
                        if (stream_weight_optimize[i]) {
                                parameters[current] = stream_weights[i];
                                current++;
                        }

                        for (int j = 0; j < stream_parameters[i].length; j++) {
                                if (stream_optimize_parameter[i][j]) {
                                        parameters[current] = stream_parameters[i][j];
                                        current++;
                                }
                        }
                }
                this.convertToConstrained();
                return parameters;
        }

        public double[] get_parameter_min() {
                this.convertToUnconstrained();

                /********
                        *       Calculate the number of parameters that will
                        *       be optimized over
                 ********/
                int p_count = 0;
                for (int i = 0; i < background_optimize_parameter.length; i++) {
                        if (background_optimize_parameter[i]) p_count++;
                }

                for (int i = 0; i < stream_optimize_parameter.length; i++) {
                        if (stream_weight_optimize[i]) p_count++;
                        for (int j = 0; j < stream_optimize_parameter[i].length; j++) {
                                if (stream_optimize_parameter[i][j]) p_count++;
                        }
                }

                double[] parameters = new double[p_count];
                int current = 0;
                for (int i = 0; i < background_parameters.length; i++) {
                        if (background_optimize_parameter[i]) {
                                parameters[current] = background_parameter_min[i];
                                current++;
                        }
                }

                for (int i = 0; i < stream_parameters.length; i++) {
                        if (stream_weight_optimize[i]) {
                                parameters[current] = stream_weight_min[i];
                                current++;
                        }

                        for (int j = 0; j < stream_parameters[i].length; j++) {
                                if (stream_optimize_parameter[i][j]) {
                                        parameters[current] = stream_parameter_min[i][j];
                                        current++;
                                }
                        }
                }
                this.convertToConstrained();
                return parameters;
        }

        public double[] get_parameter_max() {
                this.convertToUnconstrained();

                /********
                        *       Calculate the number of parameters that will
                        *       be optimized over
                 ********/
                int p_count = 0;
                for (int i = 0; i < background_optimize_parameter.length; i++) {
                        if (background_optimize_parameter[i]) p_count++;
                }

                for (int i = 0; i < stream_optimize_parameter.length; i++) {
                        if (stream_weight_optimize[i]) p_count++;
                        for (int j = 0; j < stream_optimize_parameter[i].length; j++) {
                                if (stream_optimize_parameter[i][j]) p_count++;
                        }
                }

                double[] parameters = new double[p_count];
                int current = 0;
                for (int i = 0; i < background_parameters.length; i++) {
                        if (background_optimize_parameter[i]) {
                                parameters[current] = background_parameter_max[i];
                                current++;
                        }
                }

                for (int i = 0; i < stream_parameters.length; i++) {
                        if (stream_weight_optimize[i]) {
                                parameters[current] = stream_weight_max[i];
                                current++;
                        }

                        for (int j = 0; j < stream_parameters[i].length; j++) {
                                if (stream_optimize_parameter[i][j]) {
                                        parameters[current] = stream_parameter_max[i][j];
                                        current++;
                                }
                        }
                }
                this.convertToConstrained();
                return parameters;
        }

	public void set_parameters(IntegralParameters original) {
		background_weight = original.background_weight;
		convolve = original.convolve;

		background_parameters = original.copyDoubleArray(original.background_parameters);
		background_h = original.copyDoubleArray(original.background_h);
		background_mutation_range = original.copyDoubleArray(original.background_mutation_range);
		background_parameter_min = original.copyDoubleArray(original.background_parameter_min);
		background_parameter_max = original.copyDoubleArray(original.background_parameter_max);
		background_optimize_parameter = original.copyBooleanArray(original.background_optimize_parameter);

		stream_weights = original.copyDoubleArray(original.stream_weights);
		stream_weight_h = original.copyDoubleArray(original.stream_weight_h);
		stream_weight_mutation_range = original.copyDoubleArray(original.stream_weight_mutation_range);
		stream_weight_min = original.copyDoubleArray(original.stream_weight_min);
		stream_weight_max = original.copyDoubleArray(original.stream_weight_max);
		stream_weight_optimize = original.copyBooleanArray(original.stream_weight_optimize);

		stream_parameters = original.copyDouble2Array(original.stream_parameters);
		stream_h = original.copyDouble2Array(original.stream_h);
		stream_mutation_range = original.copyDouble2Array(original.stream_mutation_range);
		stream_parameter_min = original.copyDouble2Array(original.stream_parameter_min);
		stream_parameter_max = original.copyDouble2Array(original.stream_parameter_max);
		stream_optimize_parameter = original.copyBoolean2Array(original.stream_optimize_parameter);
	}

	/*converts the unconstrained parameters to the constrained versions*/
	public void convertToConstrained() {
		double sigma[] 	= new double[stream_parameters.length];
		for (int i = 0; i < stream_parameters.length; i++) 
			sigma[i] = stream_parameters[i][4];
                double alpha    = background_parameters[0];
                double q        = background_parameters[1];
                double delta    = background_parameters[3];
		
		//convert to constrained
		for (int i = 0; i < stream_parameters.length; i++)
			sigma[i] = (0.5 + 20.0 * Math.exp(-1.0 * sigma[i] * sigma[i]));
		alpha = 3.0 - alpha * alpha;
		q = .3 + .7 * Math.exp(-1.0 * q * q);
		delta = delta * delta + .01;
	
		//make constrained parameters global	
		for (int i = 0; i < stream_parameters.length; i++)
		{
			this.stream_parameters[i][4] = sigma[i];
			this.stream_weights[i] = Math.log(stream_weights[i] * stream_weights[i]);
		}
		this.background_parameters[0] 	= alpha;
	 	this.background_parameters[1] 	= q;
	 	this.background_parameters[3] 	= delta;
	}

	/*converts the constrained parameters to the Unconstrained versions*/
	public void convertToUnconstrained() {
		double sigma[]  = new double[stream_parameters.length];
                for (int i = 0; i < stream_parameters.length; i++) {
                        sigma[i] = stream_parameters[i][4];
		}

                double alpha    = background_parameters[0];
                double q        = background_parameters[1];
                double delta    = background_parameters[3];
	
		//convert to unconstrained	
		for (int i=0; i< stream_parameters.length; i++)
			sigma[i] = Math.sqrt( -Math.log( (sigma[i] - .5) /20.0 ));

		alpha = Math.sqrt( 3.0 - alpha);
		q = Math.sqrt( -Math.log( (q - .3) /.7 ));
		if( (delta - .01) < 0)
			delta = 0;
		else
			delta = Math.sqrt( delta - .01 );
		
		//make unconstrained parameters global
                for (int i = 0; i < stream_parameters.length; i++) {
			this.stream_parameters[i][4] = sigma[i];
			this.stream_weights[i] = Math.sqrt( Math.exp( stream_weights[i] ) ); 
                }
		this.background_parameters[0]   = alpha;
                this.background_parameters[1]   = q;
                this.background_parameters[3]   = delta;
	}

	public double[] getH() {
		int total = 0;
		for (int i = 0; i < background_optimize_parameter.length; i++) {
			if (background_optimize_parameter[i]) total++;
		}

		for (int i = 0; i < stream_optimize_parameter.length; i++) {
			if (stream_weight_optimize[i]) total++;
			for (int j = 0; j < stream_optimize_parameter[i].length; j++) {
				if (stream_optimize_parameter[i][j]) total++;
			}
		}

		double[] h = new double[total];
		int current = 0;
		for (int i = 0; i < background_h.length; i++) {
			if (background_optimize_parameter[i]) {
				h[current] = background_h[i];
				current++;
			}
		}
		for (int i = 0; i < stream_h.length; i++) {
			if (stream_weight_optimize[i]) {
				h[current] = stream_weight_h[i];
				current++;
			}
			for (int j = 0; j < stream_h[i].length; j++) {
				if (stream_optimize_parameter[i][j]) {
					h[current] = stream_h[i][j];
					current++;
				}
			}
		}
		return h;
	}

	public void readParametersFromFile(String parameter_file) {
//		System.out.println("Reading paramaters from: " + parameter_file);
		try {
			BufferedReader parameterReader = new BufferedReader(new FileReader(parameter_file));

			/**
			 *	Parse background parameters
			 */
			StringTokenizer st = new StringTokenizer(parameterReader.readLine(), " :");
			st.nextToken();
			convolve = Integer.parseInt(st.nextToken());

			st = new StringTokenizer(parameterReader.readLine(), " :");
			st.nextToken();
			int number_background_parameters = Integer.parseInt(st.nextToken());
			background_parameters = new double[number_background_parameters];
			background_h = new double[number_background_parameters];
			background_mutation_range = new double[number_background_parameters];
			background_parameter_min = new double[number_background_parameters];
			background_parameter_max = new double[number_background_parameters];
			background_optimize_parameter = new boolean[number_background_parameters];

			st = new StringTokenizer(parameterReader.readLine(), " :");
			if (!st.nextToken().equals("background_weight")) throw new Exception();
			background_weight = Double.parseDouble(st.nextToken());

			background_parameters = this.readArray("background_parameters", parameterReader.readLine(), number_background_parameters);
			background_h = this.readArray("background_h", parameterReader.readLine(), number_background_parameters);
			background_mutation_range = this.readArray("background_mutation_range", parameterReader.readLine(), number_background_parameters);
			background_parameter_min = this.readArray("background_parameter_min", parameterReader.readLine(), number_background_parameters);
			background_parameter_max = this.readArray("background_parameter_max", parameterReader.readLine(), number_background_parameters);
			background_optimize_parameter = this.readBooleanArray("optimize_parameter", parameterReader.readLine(), number_background_parameters);

			st = new StringTokenizer(parameterReader.readLine(), " :,");
			st.nextToken();
			int number_streams = Integer.parseInt(st.nextToken());
			int number_stream_parameters = Integer.parseInt(st.nextToken());

			stream_weights = new double[number_streams];
			stream_weight_h = new double[number_streams];
			stream_weight_mutation_range = new double[number_streams];
			stream_weight_min = new double[number_streams];
			stream_weight_max = new double[number_streams];
			stream_weight_optimize = new boolean[number_streams];
			stream_parameters = new double[number_streams][number_stream_parameters];
			stream_h = new double[number_streams][number_stream_parameters];
			stream_mutation_range = new double[number_streams][number_stream_parameters];
			stream_parameter_min = new double[number_streams][number_stream_parameters];
			stream_parameter_max = new double[number_streams][number_stream_parameters];
			stream_optimize_parameter = new boolean[number_streams][number_stream_parameters];

			for (int i = 0; i < number_streams; i++) {
				st = new StringTokenizer(parameterReader.readLine(), " :,");
				if (!st.nextToken().equals("stream_weight")) throw new Exception();
				stream_weights[i] = Double.parseDouble(st.nextToken());

				st = new StringTokenizer(parameterReader.readLine(), " :,");
				if (!st.nextToken().equals("stream_weight_h")) throw new Exception();
				stream_weight_h[i] = Double.parseDouble(st.nextToken());

				st = new StringTokenizer(parameterReader.readLine(), " :,");
				if (!st.nextToken().equals("stream_weight_mutation_range")) throw new Exception();
				stream_weight_mutation_range[i] = Double.parseDouble(st.nextToken());

				st = new StringTokenizer(parameterReader.readLine(), " :,");
				if (!st.nextToken().equals("stream_weight_min")) throw new Exception();
				stream_weight_min[i] = Double.parseDouble(st.nextToken());

				st = new StringTokenizer(parameterReader.readLine(), " :,");
				if (!st.nextToken().equals("stream_weight_max")) throw new Exception();
				stream_weight_max[i] = Double.parseDouble(st.nextToken());

				st = new StringTokenizer(parameterReader.readLine(), " :,");
				if (!st.nextToken().equals("optimize_weight")) throw new Exception();
				String value = st.nextToken();
				if (value.equals("0") || value.equals("false")) stream_weight_optimize[i] = false;
				else stream_weight_optimize[i] = true;

				stream_parameters[i] = this.readArray("stream_parameters", parameterReader.readLine(), number_stream_parameters);
				stream_h[i] = this.readArray("stream_h", parameterReader.readLine(), number_stream_parameters);
				stream_mutation_range[i] = this.readArray("stream_mutation_range", parameterReader.readLine(), number_stream_parameters);
				stream_parameter_min[i] = this.readArray("stream_parameter_min", parameterReader.readLine(), number_stream_parameters);
				stream_parameter_max[i] = this.readArray("stream_parameter_max", parameterReader.readLine(), number_stream_parameters);
				stream_optimize_parameter[i] = this.readBooleanArray("optimize_parameter", parameterReader.readLine(), number_stream_parameters);
			}
			parameterReader.close();
		} catch (Exception e) {
			System.err.println("Error reading from parameter file: " + parameter_file);
			System.err.println("\tException: " + e);
			e.printStackTrace();
		}
	}

	public double[] readArray(String array_name, String line, int size) throws Exception {
		StringTokenizer st = new StringTokenizer(line, " :,");
		if (!st.nextToken().equals(array_name)) throw new Exception();
		double[] values = new double[size];
		for (int i = 0; i < size; i++) values[i] = Double.parseDouble(st.nextToken());
		return values;
	}

	public boolean[] readBooleanArray(String array_name, String line, int size) throws Exception {
		StringTokenizer st = new StringTokenizer(line, " :,");
		if (!st.nextToken().equals(array_name)) throw new Exception();
		boolean[] values = new boolean[size];
		for (int i = 0; i < size; i++) {
			String value = st.nextToken();

			if (value.equals("0") || value.equals("false")) {
				values[i] = false;
			} else {
				values[i] = true;
			}
		}
		return values;
	}

	public void printParameters(OutputStream out) {
		try {
			DataOutputStream outfile = new DataOutputStream(out);

			outfile.writeBytes("convolve: " + convolve + "\n");
			outfile.writeBytes("number_parameters: " + background_parameters.length + "\n");
			outfile.writeBytes("background_weight: " + background_weight + "\n");
			writeArray("background_parameters", background_parameters, outfile);
			writeArray("background_h", background_h, outfile);
			writeArray("background_mutation_range", background_mutation_range, outfile);
			writeArray("background_parameter_min", background_parameter_min, outfile);
			writeArray("background_parameter_max", background_parameter_max, outfile);
			writeArray("optimize_parameter", background_optimize_parameter, outfile);

			outfile.writeBytes("number_streams: " + stream_parameters.length + ", " + stream_parameters[0].length + "\n");
			for (int i = 0; i < stream_parameters.length; i++) {
				outfile.writeBytes("stream_weight: " + stream_weights[i] + "\n");
				outfile.writeBytes("stream_weight_h: " + stream_weight_h[i] + "\n");
				outfile.writeBytes("stream_weight_mutation_range: " + stream_weight_mutation_range[i] + "\n");
				outfile.writeBytes("stream_weight_min: " + stream_weight_min[i] + "\n");
				outfile.writeBytes("stream_weight_max: " + stream_weight_max[i] + "\n");
				outfile.writeBytes("optimize_weight: " + stream_weight_optimize[i] + "\n");

				writeArray("stream_parameters", stream_parameters[i], outfile);
				writeArray("stream_h", stream_h[i], outfile);
				writeArray("stream_mutation_range", stream_mutation_range[i], outfile);
				writeArray("stream_parameter_min", stream_parameter_min[i], outfile);
				writeArray("stream_parameter_max", stream_parameter_max[i], outfile);
				writeArray("optimize_parameter", stream_optimize_parameter[i], outfile);
			}
		} catch (Exception e) {
			System.err.println("Error printing parameters to OutputStream: " + e);
			e.printStackTrace();
			System.exit(0);
		}
	}

	public void writeArray(String name, double[] array, DataOutputStream outfile) throws Exception {
		int i;
		outfile.writeBytes(name + ": ");
		for (i = 0; i < array.length-1; i++) outfile.writeBytes(array[i] + ", ");
		outfile.writeBytes(array[i] + "\n");
	}

	public void writeArray(String name, boolean[] array, DataOutputStream outfile) throws Exception {
		int i;
		outfile.writeBytes(name + ": ");
		for (i = 0; i < array.length-1; i++) outfile.writeBytes(array[i] + ", ");
		outfile.writeBytes(array[i] + "\n");
	}

	public void printParameters(String file) {
		try {
			FileOutputStream outfile = new FileOutputStream(file);
			printParameters(outfile);
			outfile.close();
		} catch (Exception e) {
			System.err.println("Error printing parameters to FileWriter: " + e);
			e.printStackTrace();
			System.exit(0);
		}
	}
}
