package gmle.salsa.astronomy;

// Import declarations generated by the SALSA compiler, do not modify.
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import salsa_lite.core.language.Actor;
import salsa_lite.core.language.ActorReference;
import salsa_lite.core.language.IdentifierGenerator;
import salsa_lite.core.language.Message;
import salsa_lite.core.language.Token;
import salsa_lite.core.language.exceptions.CurrentContinuationException;
import salsa_lite.core.language.exceptions.MessageHandlerNotFoundException;

import salsa_lite.core.language.LiteActorState;

import salsa_lite.core.naming.UAN;
import salsa_lite.core.naming.UAL;
import salsa_lite.core.naming.MalformedUALException;
import salsa_lite.core.naming.MalformedUANException;

// End SALSA compiler generated import delcarations.

import java.io.*;
import Jama.*;
import java.util.Vector;
import java.util.StringTokenizer;
import gmle.salsa.framework.GradientDescent;
import gmle.salsa.framework.GeneticSearchProbabilistic;
import gmle.salsa.framework.MLEvaluator;
import gmle.salsa.framework.Simplex;

public class MaxLikelihoodState extends LiteActorState  {
	public MaxLikelihoodState(UAN uan) {
		super(uan);
		self = new MaxLikelihood(getUAN());
	}

	public MaxLikelihoodState(UAL ual) {
		super(ual);
		self = new MaxLikelihood(getUAL());
	}

	public MaxLikelihoodState(String identifier) {
		super(identifier);
		self = new MaxLikelihood(getUAL());
	}

	public Object invokeMessage(String methodName, Object[] arguments) throws CurrentContinuationException, MessageHandlerNotFoundException, Exception {
		if (methodName.equals("act") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof String[])) {
			act((String[])arguments[0]);
			return null;
		}  else if (methodName.equals("evaluate") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof IntegralParameters)) {
			return new Double(evaluate((IntegralParameters)arguments[0]));
		}  else if (methodName.equals("get_error_values") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof IntegralParameters)) {
			get_error_values((IntegralParameters)arguments[0]);
			return null;
		}  else if (methodName.equals("calculate_hessian") && arguments.length == 0) {
			calculate_hessian();
			return null;
		}  else if (methodName.equals("perturb_parameter") && arguments.length == 3 && (arguments[0] == null || arguments[0] instanceof IntegralParameters) && (arguments[1] == null || arguments[1] instanceof Integer) && (arguments[2] == null || arguments[2] instanceof Double)) {
			return perturb_parameter((IntegralParameters)arguments[0], ((Integer)arguments[1]).intValue(), ((Double)arguments[2]).doubleValue());
		}  else if (methodName.equals("set_Hij") && arguments.length == 6 && (arguments[0] == null || arguments[0] instanceof Integer) && (arguments[1] == null || arguments[1] instanceof Integer) && (arguments[2] == null || arguments[2] instanceof Double) && (arguments[3] == null || arguments[3] instanceof Double) && (arguments[4] == null || arguments[4] instanceof Double) && (arguments[5] == null || arguments[5] instanceof Double)) {
			set_Hij(((Integer)arguments[0]).intValue(), ((Integer)arguments[1]).intValue(), ((Double)arguments[2]).doubleValue(), ((Double)arguments[3]).doubleValue(), ((Double)arguments[4]).doubleValue(), ((Double)arguments[5]).doubleValue());
			return null;
		}  else if (methodName.equals("set_Hij") && arguments.length == 8 && (arguments[0] == null || arguments[0] instanceof Integer) && (arguments[1] == null || arguments[1] instanceof Integer) && (arguments[2] == null || arguments[2] instanceof Double) && (arguments[3] == null || arguments[3] instanceof Double) && (arguments[4] == null || arguments[4] instanceof Double) && (arguments[5] == null || arguments[5] instanceof Double) && (arguments[6] == null || arguments[6] instanceof Double) && (arguments[7] == null || arguments[7] instanceof Double)) {
			set_Hij(((Integer)arguments[0]).intValue(), ((Integer)arguments[1]).intValue(), ((Double)arguments[2]).doubleValue(), ((Double)arguments[3]).doubleValue(), ((Double)arguments[4]).doubleValue(), ((Double)arguments[5]).doubleValue(), ((Double)arguments[6]).doubleValue(), ((Double)arguments[7]).doubleValue());
			return null;
		}  else if (methodName.equals("paramSweep") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof IntegralParameters)) {
			paramSweep((IntegralParameters)arguments[0]);
			return null;
		}  else if (methodName.equals("paramsToUnconst") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof IntegralParameters)) {
			return paramsToUnconst((IntegralParameters)arguments[0]);
		}  else if (methodName.equals("setProb") && arguments.length == 5 && (arguments[0] == null || arguments[0] instanceof Integer) && (arguments[1] == null || arguments[1] instanceof Integer) && (arguments[2] == null || arguments[2] instanceof IntegralParameters) && (arguments[3] == null || arguments[3] instanceof IntegralParameters) && (arguments[4] == null || arguments[4] instanceof Double)) {
			setProb(((Integer)arguments[0]).intValue(), ((Integer)arguments[1]).intValue(), (IntegralParameters)arguments[2], (IntegralParameters)arguments[3], ((Double)arguments[4]).doubleValue());
			return null;
		}  else if (methodName.equals("makeParamSweepFile") && arguments.length == 0) {
			makeParamSweepFile();
			return null;
		}  else if (methodName.equals("construct") && arguments.length == 0) {
			construct();
			return null;
		} else {
			return super.invokeMessage(methodName, arguments);
		}
	}

	void construct() {}

	String theaters_file;
	int num_stars = 250000;
	int number_sweep_points = 25;
	int number_parameters = 10;
	public void act(String[] arguments) {
		if (arguments.length!=4) {
			System.err.println("Invalid Arguments.");
			System.err.println("Usage:");
			System.err.println("\tjava gmle.salsa.astronomy.MaxLikelihood <search_type> <parameter_file> <star_file> <volume_file>");
			System.err.println("Search Types:");
			System.err.println("\tgd = gradient descent");
			System.err.println("\tcgd = conjugate gradient descent");
			System.err.println("\thcgd = hessian conjugate gradient descent");
			System.err.println("\tseparation = separation function");
			System.err.println("\tmake_test_data = generate new test data");
			System.err.println("\terrors = run error analysis");
			System.err.println("\tparameter_sweep = run parameter sweep");
			System.err.println("Options:");
			System.err.println("-Doutfile=<output file>");
			System.err.println("-Dtheaters=<nameserver and theater file>");
			System.exit(0);
		}
		String search_type = arguments[0];
		String initial_parameters_file = arguments[1];
		String star_file = arguments[2];
		String volume_file = arguments[3];
		String theaters_file = System.getProperty("theaters");
		String output_file = System.getProperty("outfile");
		IntegralParameters integral_parameters = new IntegralParameters(initial_parameters_file);
		Object[] init_params = { star_file, volume_file, initial_parameters_file };
		if (search_type.equals("gd")||search_type.equals("cgd")||search_type.equals("hcgd")) {
			double[] initial_parameters = integral_parameters.get_parameters();
			double[] step_sizes = integral_parameters.getH();
			GradientDescent gradient_descent;
			if (theaters_file==null) {
				gradient_descent = GradientDescent.construct(self, null, null, -1, new Object[]{"gmle.salsa.astronomy.AstronomyEvaluator", new Integer(4)});
			}
			else {
				gradient_descent = GradientDescent.construct(self, null, null, -1, new Object[]{"gmle.salsa.astronomy.AstronomyEvaluator", theaters_file});
			}
			Token init_token = new Token("init_token");
			{
				// token init_token = gradient_descent<-initialize(init_params)
				{
					Message __message = new Message( self, gradient_descent, "initialize", new Object[]{ init_params }, null, init_token);
					__messages.add( __message );
				}
			}
			if (arguments[0].equals("gd")) {
				{
					// gradient_descent<-gradient_descent(initial_parameters, step_sizes)
					{
						Message __message = new Message( self, gradient_descent, "gradient_descent", new Object[]{ initial_parameters, step_sizes }, null, null);
						Object[] _propertyInfo = { init_token };
						__message.setProperty( "waitfor", _propertyInfo );
						__messages.add( __message );
					}
				}
			}
			else if (arguments[0].equals("cgd")) {
				{
					// gradient_descent<-conjugate_gradient_descent(initial_parameters, step_sizes)
					{
						Message __message = new Message( self, gradient_descent, "conjugate_gradient_descent", new Object[]{ initial_parameters, step_sizes }, null, null);
						Object[] _propertyInfo = { init_token };
						__message.setProperty( "waitfor", _propertyInfo );
						__messages.add( __message );
					}
				}
			}
			else if (arguments[0].equals("hcgd")) {
				{
					// gradient_descent<-hessian_conjugate_gradient_descent(initial_parameters, step_sizes)
					{
						Message __message = new Message( self, gradient_descent, "hessian_conjugate_gradient_descent", new Object[]{ initial_parameters, step_sizes }, null, null);
						Object[] _propertyInfo = { init_token };
						__message.setProperty( "waitfor", _propertyInfo );
						__messages.add( __message );
					}
				}
			}
		}
		else if (search_type.equals("gsp")) {
			GeneticSearchProbabilistic gsp;
			if (theaters_file==null) {
				gsp = GeneticSearchProbabilistic.construct(self, null, null, -1, new Object[]{"gmle.salsa.astronomy.AstronomyEvaluator", 4});
			}
			else {
				gsp = GeneticSearchProbabilistic.construct(self, null, null, -1, new Object[]{"gmle.salsa.astronomy.AstronomyEvaluator", theaters_file});
			}
			{
				Token token_3_0 = new Token();
				// gsp<-initialize(init_params)
				{
					Message __message = new Message( self, gsp, "initialize", new Object[]{ init_params }, null, token_3_0);
					__messages.add( __message );
				}
				// gsp<-genetic_search(integral_parameters.get_parameter_min(), integral_parameters.get_parameter_max(), 100, new Long(25000), 0.8)
				{
					Message __message = new Message( self, gsp, "genetic_search", new Object[]{ integral_parameters.get_parameter_min(), integral_parameters.get_parameter_max(), new Integer(100), new Long(25000), new Double(0.8) }, token_3_0, null);
					__messages.add( __message );
				}
			}
		}
		else if (search_type.equals("simplex")) {
			double[][] initial_parameters = integral_parameters.get_simplex_parameters();
			Simplex simplex;
			if (theaters_file==null) {
				simplex = Simplex.construct(self, null, null, -1, new Object[]{"gmle.salsa.astronomy.AstronomyEvaluator", 4});
			}
			else {
				simplex = Simplex.construct(self, null, null, -1, new Object[]{"gmle.salsa.astronomy.AstronomyEvaluator", theaters_file});
			}
			{
				Token token_3_0 = new Token();
				// simplex<-initialize(init_params)
				{
					Message __message = new Message( self, simplex, "initialize", new Object[]{ init_params }, null, token_3_0);
					__messages.add( __message );
				}
				// simplex<-simplex(initial_parameters)
				{
					Message __message = new Message( self, simplex, "simplex", new Object[]{ initial_parameters }, token_3_0, null);
					__messages.add( __message );
				}
			}
		}
		else if (search_type.equals("errors")) {
			if (theaters_file!=null) {
				evaluator = MLEvaluator.construct(self, null, null, -1, new Object[]{"gmle.salsa.astronomy.AstronomyEvaluator", theaters_file});
			}
			else {
				evaluator = MLEvaluator.construct(self, null, null, -1, new Object[]{"gmle.salsa.astronomy.AstronomyEvaluator", 4});
			}
			{
				Token token_3_0 = new Token();
				Token token_3_1 = new Token();
				// evaluator<-initialize(init_params)
				{
					Message __message = new Message( self, evaluator, "initialize", new Object[]{ init_params }, null, token_3_0);
					__messages.add( __message );
				}
				// evaluator<-init_data()
				{
					Message __message = new Message( self, evaluator, "init_data", new Object[]{  }, token_3_0, token_3_1);
					__messages.add( __message );
				}
				// get_error_values()
				{
					Message __message = new Message( self, self, "get_error_values", new Object[]{  }, token_3_1, null);
					__messages.add( __message );
				}
			}
		}
		else if (search_type.equals("parameter_sweep")) {
			if (theaters_file!=null) {
				evaluator = MLEvaluator.construct(self, null, null, -1, new Object[]{"gmle.salsa.astronomy.AstronomyEvaluator", theaters_file});
			}
			else {
				evaluator = MLEvaluator.construct(self, null, null, -1, new Object[]{"gmle.salsa.astronomy.AstronomyEvaluator", 4});
			}
			{
				Token token_3_0 = new Token();
				Token token_3_1 = new Token();
				// evaluator<-initialize(init_params)
				{
					Message __message = new Message( self, evaluator, "initialize", new Object[]{ init_params }, null, token_3_0);
					__messages.add( __message );
				}
				// evaluator<-init_data()
				{
					Message __message = new Message( self, evaluator, "init_data", new Object[]{  }, token_3_0, token_3_1);
					__messages.add( __message );
				}
				// paramSweep()
				{
					Message __message = new Message( self, self, "paramSweep", new Object[]{  }, token_3_1, null);
					__messages.add( __message );
				}
			}
		}
		else if (search_type.equals("separation")) {
			AstronomyEvaluator evaluator = AstronomyEvaluator.construct(self, null, null, -1, new Object[]{});
			{
				Token token_3_0 = new Token();
				Token token_3_1 = new Token();
				// evaluator<-initialize(init_params, 0, 1)
				{
					Message __message = new Message( self, evaluator, "initialize", new Object[]{ init_params, new Integer(0), new Integer(1) }, null, token_3_0);
					__messages.add( __message );
				}
				// evaluator<-integral_function()
				{
					Message __message = new Message( self, evaluator, "integral_function", new Object[]{  }, token_3_0, token_3_1);
					__messages.add( __message );
				}
				// evaluator<-separation_function(integral_parameters)
				{
					Message __message = new Message( self, evaluator, "separation_function", new Object[]{ integral_parameters }, token_3_1, null);
					__messages.add( __message );
				}
			}
		}
		else if (search_type.equals("make_test_data")) {
			AstronomyEvaluator evaluator = AstronomyEvaluator.construct(self, null, null, -1, new Object[]{});
			{
				Token token_3_0 = new Token();
				Token token_3_1 = new Token();
				// evaluator<-initialize(init_params, 0, 1)
				{
					Message __message = new Message( self, evaluator, "initialize", new Object[]{ init_params, new Integer(0), new Integer(1) }, null, token_3_0);
					__messages.add( __message );
				}
				// evaluator<-integral_function()
				{
					Message __message = new Message( self, evaluator, "integral_function", new Object[]{  }, token_3_0, token_3_1);
					__messages.add( __message );
				}
				// evaluator<-make_test_data(integral_parameters)
				{
					Message __message = new Message( self, evaluator, "make_test_data", new Object[]{ integral_parameters }, token_3_1, null);
					__messages.add( __message );
				}
			}
		}
	}
	MLEvaluator evaluator;
	public double evaluate(IntegralParameters integral_parameters) {
		{
			// evaluator<-evaluate(integral_parameters.get_parameters())
			{
				Message __message = new Message( self, evaluator, "evaluate", new Object[]{ integral_parameters.get_parameters() }, null, currentMessage.getContinuationToken());
				__message.setJoinPosition(currentMessage.getJoinPosition());
				__messages.add( __message );
			}
			throw new CurrentContinuationException();
		}
	}
	public void get_error_values(IntegralParameters integral_parameters) {
		double hi;
		double hj;
		H = new double[number_parameters][number_parameters];
		IntegralParameters parameters = (IntegralParameters)integral_parameters.copy();
		Token wait_token = new Token("wait_token");
		{
			// token wait_token = standardOutput<-println("Getting error values.")
			{
				Message __message = new Message( self, standardOutput, "println", new Object[]{ "Getting error values." }, null, wait_token);
				__messages.add( __message );
			}
		}
		for (int i = 0; i<number_parameters; i++){
			if (i<5) hi = 0.2;
			else if (i==8) hi = 0.02;
			else if (i==7) hi = 0.005;
			else if (i==5) hi = 0.1;
			else hi = 0.2;
			for (int j = 0; j<=i; j++){
				if (i<5) hj = 0.2;
				else if (i==8) hj = 0.02;
				else if (i==7) hj = 0.005;
				else hj = 0.1;
				if ((i==6)||(i==9)||(j==6)||(j==9)) {
					H[i][j] = 0.0;
					H[j][i] = 0.0;
				}
				else if (i==j) {
					Token params = new Token("params");
					Token Hij1 = new Token("Hij1");
					Token Hij2 = new Token("Hij2");
					Token Hij3 = new Token("Hij3");
					{
						Token token_5_0 = new Token();
						Token token_5_2 = new Token();
						Token token_5_4 = new Token();
						Token token_5_6 = new Token();
						Token token_5_8 = new Token();
						Token token_5_10 = new Token();
						Token token_5_12 = new Token();
						// standardOutput<-println("Starting iteration : ["+i+", "+j+"]")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "Starting iteration : ["+i+", "+j+"]" }, null, token_5_0);
							Object[] _propertyInfo = { wait_token };
							__message.setProperty( "waitfor", _propertyInfo );
							__messages.add( __message );
						}
						// token params = perturb_parameter(parameters.copy(), new Integer(j), new Double(2.0*hj))
						{
							Message __message = new Message( self, self, "perturb_parameter", new Object[]{ parameters.copy(), new Integer(j), new Double(2.0*hj) }, token_5_0, params);
							__messages.add( __message );
						}
						// standardOutput<-println("\ti == j, perturbing parameters 1/3")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti == j, perturbing parameters 1/3" }, params, token_5_2);
							__messages.add( __message );
						}
						// token Hij1 = evaluate(params)
						{
							Message __message = new Message( self, self, "evaluate", new Object[]{ params }, token_5_2, Hij1);
							__messages.add( __message );
						}
						// standardOutput<-println("\ti == j, evaluation 1/3 complete.")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti == j, evaluation 1/3 complete." }, Hij1, token_5_4);
							__messages.add( __message );
						}
						// params = perturb_parameter(params, new Integer(j), new Double(-2.0*hj))
						Token params_next = new Token("perturb_parameter_next");
						{
							Message __message = new Message( self, self, "perturb_parameter", new Object[]{ params, new Integer(j), new Double(-2.0*hj) }, token_5_4, params_next);
							__messages.add( __message );
						}
						params = params_next;
						// standardOutput<-println("\ti == j, perturbing parameters 2/3")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti == j, perturbing parameters 2/3" }, params_next, token_5_6);
							__messages.add( __message );
						}
						// token Hij2 = evaluate(params)
						{
							Message __message = new Message( self, self, "evaluate", new Object[]{ params }, token_5_6, Hij2);
							__messages.add( __message );
						}
						// standardOutput<-println("\ti == j, evaluation 2/3 complete.")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti == j, evaluation 2/3 complete." }, Hij2, token_5_8);
							__messages.add( __message );
						}
						// params = perturb_parameter(params, new Integer(j), new Double(-2.0*hj))
						params_next = new Token("perturb_parameter_next");
						{
							Message __message = new Message( self, self, "perturb_parameter", new Object[]{ params, new Integer(j), new Double(-2.0*hj) }, token_5_8, params_next);
							__messages.add( __message );
						}
						params = params_next;
						// standardOutput<-println("\ti == j, perturbing parameters 3/3")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti == j, perturbing parameters 3/3" }, params_next, token_5_10);
							__messages.add( __message );
						}
						// token Hij3 = evaluate(params)
						{
							Message __message = new Message( self, self, "evaluate", new Object[]{ params }, token_5_10, Hij3);
							__messages.add( __message );
						}
						// standardOutput<-println("\ti == j, evaluation 3/3 complete.")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti == j, evaluation 3/3 complete." }, Hij3, token_5_12);
							__messages.add( __message );
						}
						// wait_token = new Integer(i)new Integer(j)new Double(hj)Hij1Hij2Hij3set_Hij:((wait_token))
						Token wait_token_next = new Token("set_Hij_next");
						{
							Message __message = new Message( self, self, "set_Hij", new Object[]{ new Integer(i), new Integer(j), new Double(hj), Hij1, Hij2, Hij3 }, token_5_12, wait_token_next);
							Object[] _propertyInfo = { wait_token };
							__message.setProperty( "waitfor", _propertyInfo );
							__messages.add( __message );
						}
						wait_token = wait_token_next;
					}
					{
						// wait_token = standardOutput<-println("Completed iteration : ["+i+", "+j+"]")
						Token wait_token_next = new Token("<-_next");
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "Completed iteration : ["+i+", "+j+"]" }, null, wait_token_next);
							Object[] _propertyInfo = { wait_token };
							__message.setProperty( "waitfor", _propertyInfo );
							__messages.add( __message );
						}
						wait_token = wait_token_next;
					}
				}
				else {
					Token params = new Token("params");
					Token Hij1 = new Token("Hij1");
					Token Hij2 = new Token("Hij2");
					Token Hij3 = new Token("Hij3");
					Token Hij4 = new Token("Hij4");
					{
						Token token_5_0 = new Token();
						Token token_5_2 = new Token();
						Token token_5_4 = new Token();
						Token token_5_6 = new Token();
						Token token_5_8 = new Token();
						Token token_5_10 = new Token();
						Token token_5_12 = new Token();
						Token token_5_14 = new Token();
						Token token_5_16 = new Token();
						Token token_5_18 = new Token();
						Token token_5_20 = new Token();
						// standardOutput<-println("Starting iteration : ["+i+", "+j+"]")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "Starting iteration : ["+i+", "+j+"]" }, null, token_5_0);
							Object[] _propertyInfo = { wait_token };
							__message.setProperty( "waitfor", _propertyInfo );
							__messages.add( __message );
						}
						// token params = perturb_parameter(parameters.copy(), new Integer(j), new Double(hj))
						{
							Message __message = new Message( self, self, "perturb_parameter", new Object[]{ parameters.copy(), new Integer(j), new Double(hj) }, token_5_0, params);
							__messages.add( __message );
						}
						// standardOutput<-println("\ti != j, perturbing parameters 1/6")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti != j, perturbing parameters 1/6" }, params, token_5_2);
							__messages.add( __message );
						}
						// params = perturb_parameter(params, new Integer(i), new Double(hi))
						Token params_next = new Token("perturb_parameter_next");
						{
							Message __message = new Message( self, self, "perturb_parameter", new Object[]{ params, new Integer(i), new Double(hi) }, token_5_2, params_next);
							__messages.add( __message );
						}
						params = params_next;
						// standardOutput<-println("\ti != j, perturbing parameters 2/6")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti != j, perturbing parameters 2/6" }, params_next, token_5_4);
							__messages.add( __message );
						}
						// token Hij1 = evaluate(params)
						{
							Message __message = new Message( self, self, "evaluate", new Object[]{ params }, token_5_4, Hij1);
							__messages.add( __message );
						}
						// standardOutput<-println("\ti != j, evaluation 1/4 complete.")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti != j, evaluation 1/4 complete." }, Hij1, token_5_6);
							__messages.add( __message );
						}
						// params = perturb_parameter(params, new Integer(j), new Double(-2.0*hj))
						params_next = new Token("perturb_parameter_next");
						{
							Message __message = new Message( self, self, "perturb_parameter", new Object[]{ params, new Integer(j), new Double(-2.0*hj) }, token_5_6, params_next);
							__messages.add( __message );
						}
						params = params_next;
						// standardOutput<-println("\ti != j, perturbing parameters 3/6")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti != j, perturbing parameters 3/6" }, params_next, token_5_8);
							__messages.add( __message );
						}
						// token Hij2 = evaluate(params)
						{
							Message __message = new Message( self, self, "evaluate", new Object[]{ params }, token_5_8, Hij2);
							__messages.add( __message );
						}
						// standardOutput<-println("\ti != j, evaluation 2/4 complete.")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti != j, evaluation 2/4 complete." }, Hij2, token_5_10);
							__messages.add( __message );
						}
						// params = perturb_parameter(params, new Integer(j), new Double(2.0*hj))
						params_next = new Token("perturb_parameter_next");
						{
							Message __message = new Message( self, self, "perturb_parameter", new Object[]{ params, new Integer(j), new Double(2.0*hj) }, token_5_10, params_next);
							__messages.add( __message );
						}
						params = params_next;
						// standardOutput<-println("\ti != j, perturbing parameters 4/6")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti != j, perturbing parameters 4/6" }, params_next, token_5_12);
							__messages.add( __message );
						}
						// params = perturb_parameter(params, new Integer(i), new Double(-2.0*hi))
						params_next = new Token("perturb_parameter_next");
						{
							Message __message = new Message( self, self, "perturb_parameter", new Object[]{ params, new Integer(i), new Double(-2.0*hi) }, token_5_12, params_next);
							__messages.add( __message );
						}
						params = params_next;
						// standardOutput<-println("\ti != j, perturbing parameters 5/6")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti != j, perturbing parameters 5/6" }, params_next, token_5_14);
							__messages.add( __message );
						}
						// token Hij3 = evaluate(params)
						{
							Message __message = new Message( self, self, "evaluate", new Object[]{ params }, token_5_14, Hij3);
							__messages.add( __message );
						}
						// standardOutput<-println("\ti != j, evaluation 3/4 complete.")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti != j, evaluation 3/4 complete." }, Hij3, token_5_16);
							__messages.add( __message );
						}
						// params = perturb_parameter(params, new Integer(j), new Double(-2.0*hj))
						params_next = new Token("perturb_parameter_next");
						{
							Message __message = new Message( self, self, "perturb_parameter", new Object[]{ params, new Integer(j), new Double(-2.0*hj) }, token_5_16, params_next);
							__messages.add( __message );
						}
						params = params_next;
						// standardOutput<-println("\ti != j, perturbing parameters 6/6")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti != j, perturbing parameters 6/6" }, params_next, token_5_18);
							__messages.add( __message );
						}
						// token Hij4 = evaluate(params)
						{
							Message __message = new Message( self, self, "evaluate", new Object[]{ params }, token_5_18, Hij4);
							__messages.add( __message );
						}
						// standardOutput<-println("\ti != j, evaluation 4/4 complete.")
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "\ti != j, evaluation 4/4 complete." }, Hij4, token_5_20);
							__messages.add( __message );
						}
						// wait_token = new Integer(i)new Integer(j)new Double(hi)new Double(hj)Hij1Hij2Hij3Hij4set_Hij:((wait_token))
						Token wait_token_next = new Token("set_Hij_next");
						{
							Message __message = new Message( self, self, "set_Hij", new Object[]{ new Integer(i), new Integer(j), new Double(hi), new Double(hj), Hij1, Hij2, Hij3, Hij4 }, token_5_20, wait_token_next);
							Object[] _propertyInfo = { wait_token };
							__message.setProperty( "waitfor", _propertyInfo );
							__messages.add( __message );
						}
						wait_token = wait_token_next;
					}
					{
						// wait_token = standardOutput<-println("Completed iteration : ["+i+", "+j+"]")
						Token wait_token_next = new Token("<-_next");
						{
							Message __message = new Message( self, standardOutput, "println", new Object[]{ "Completed iteration : ["+i+", "+j+"]" }, null, wait_token_next);
							Object[] _propertyInfo = { wait_token };
							__message.setProperty( "waitfor", _propertyInfo );
							__messages.add( __message );
						}
						wait_token = wait_token_next;
					}
				}
			}
		}
		{
			// ()calculate_hessian:((wait_token))
			{
				Message __message = new Message( self, self, "calculate_hessian", new Object[]{  }, null, null);
				Object[] _propertyInfo = { wait_token };
				__message.setProperty( "waitfor", _propertyInfo );
				__messages.add( __message );
			}
		}
	}
	public void calculate_hessian() {
		System.err.println("calculating hessian.");
		Matrix hessian;
		if (true) {
			int optimized[] = { 0, 1, 2, 3, 4, 5, 7, 8 };
			Matrix matH = new Matrix(H);
			hessian = matH.getMatrix(optimized, optimized);
		}
		else {
			hessian = new Matrix(H);
		}
		System.err.println("Hessian:");
		hessian.print(10, 5);
		Matrix sigmaN = hessian.inverse();
		Matrix sigma = sigmaN.times(1/((double)num_stars));
		System.err.println("\nSigma:");
		sigma.print(10, 5);
		try {
			PrintWriter pw = new PrintWriter(new FileOutputStream("hessian.txt"));
			hessian.print(pw, 10, 6);
			pw.close();
		}
		catch (Exception e) {
			System.err.println("Error in creating hessian.txt "+e);
			e.printStackTrace();
			System.exit(0);
		}

		try {
			PrintWriter pw = new PrintWriter(new FileOutputStream("sigma.txt"));
			sigma.print(pw, 10, 6);
			pw.close();
		}
		catch (Exception e) {
			System.err.println("Error in creating sigma.txt "+e);
			e.printStackTrace();
			System.exit(0);
		}

	}
	public IntegralParameters perturb_parameter(IntegralParameters parameters, int i, double perturbation) {
		parameters.convertToUnconstrained();
		if (i<5) {
			parameters.stream_parameters[0][i] += perturbation;
		}
		else if (i==5) {
			parameters.stream_weights[0] += perturbation;
		}
		else if ((i>5)&&(i<10)) {
			parameters.background_parameters[i-6] += perturbation;
		}
		parameters.convertToConstrained();
		return parameters;
	}
	double[][] H;
	public void set_Hij(int i, int j, double hj, double Hij1, double Hij2, double Hij3) {
		double Hij = Hij1-(2.0*Hij2)+Hij3;
		H[i][j] = Hij/(4.0*hj*hj);
		H[j][i] = H[i][j];
		System.err.println("H["+i+"]["+j+"]="+H[i][j]);
	}
	public void set_Hij(int i, int j, double hi, double hj, double Hij1, double Hij2, double Hij3, double Hij4) {
		double Hij = Hij1-Hij2-Hij3+Hij4;
		H[i][j] = Hij/(4.0*hi*hj);
		H[j][i] = H[i][j];
		System.err.println("H["+i+"]["+j+"]="+H[i][j]);
	}
	double[][] constrainedFileArray;
	double[][] unconstrainedFileArray;
	public void paramSweep(IntegralParameters integral_parameters) {
		IntegralParameters parameters = (IntegralParameters)integral_parameters.copy();
		int numpoints = number_sweep_points;
		constrainedFileArray = new double[numpoints*(number_parameters)][2];
		unconstrainedFileArray = new double[numpoints*(number_parameters)][2];
		double change[] = new double[number_parameters];
		Token wait_token = new Token("wait_token");
		{
			// token wait_token = standardOutput<-println("\nBeginning parameter sweep.")
			{
				Message __message = new Message( self, standardOutput, "println", new Object[]{ "\nBeginning parameter sweep." }, null, wait_token);
				__messages.add( __message );
			}
		}
		for (int i = 0; i<number_parameters; i++){
			for (int j = 0; j<numpoints; j++){
				if (i<1) {
					change[i] = 1.0*(double)(j)/(double)(numpoints)-0.5;
				}
				else if (i<2) {
					change[i] = 1.0*(double)(j)/(double)(numpoints)-0.5;
				}
				else if (i<3) {
					change[i] = 1.0*(double)(j)/(double)(numpoints)-0.5;
				}
				else if (i<4) {
					change[i] = 1.0*(double)(j)/(double)(numpoints)-0.5;
				}
				else if (i<5) {
					change[i] = 1.0*(double)(j)/(double)(numpoints)-0.5;
				}
				else if (i<6) {
					change[i] = 1.0*(double)(j)/(double)(numpoints)-0.5;
				}
				else if (i<7) {
					change[i] = 1.0*(double)(j)/(double)(numpoints)-0.5;
				}
				else if (i<8) {
					change[i] = 1.0*(double)(j)/(double)(numpoints)-0.5;
				}
				else if (i<9) {
					change[i] = 1.0*(double)(j)/(double)(numpoints)-0.5;
				}
				else if (i<10) {
					change[i] = 1.0*(double)(j)/(double)(numpoints)-0.75;
				}
				Token params = new Token("params");
				Token prob = new Token("prob");
				{
					// token params = parameters.copy()new Integer(i)new Double(change[i])perturb_parameter:((wait_token))
					{
						Message __message = new Message( self, self, "perturb_parameter", new Object[]{ parameters.copy(), new Integer(i), new Double(change[i]) }, null, params);
						Object[] _propertyInfo = { wait_token };
						__message.setProperty( "waitfor", _propertyInfo );
						__messages.add( __message );
					}
					// token prob = evaluate(params)
					{
						Message __message = new Message( self, self, "evaluate", new Object[]{ params }, params, prob);
						__messages.add( __message );
					}
				}
				Token unconstrainedParams = new Token("unconstrainedParams");
				{
					// token unconstrainedParams = paramsToUnconst(params)
					{
						Message __message = new Message( self, self, "paramsToUnconst", new Object[]{ params }, null, unconstrainedParams);
						__messages.add( __message );
					}
				}
				{
					// wait_token = setProb(new Integer(i), new Integer(j), params, unconstrainedParams, prob)
					Token wait_token_next = new Token("setProb_next");
					{
						Message __message = new Message( self, self, "setProb", new Object[]{ new Integer(i), new Integer(j), params, unconstrainedParams, prob }, null, wait_token_next);
						__messages.add( __message );
					}
					wait_token = wait_token_next;
				}
			}
		}
		{
			// ()makeParamSweepFile:((wait_token))
			{
				Message __message = new Message( self, self, "makeParamSweepFile", new Object[]{  }, null, null);
				Object[] _propertyInfo = { wait_token };
				__message.setProperty( "waitfor", _propertyInfo );
				__messages.add( __message );
			}
		}
	}
	public IntegralParameters paramsToUnconst(IntegralParameters parameters) {
		IntegralParameters unconst = (IntegralParameters)parameters.copy();
		unconst.convertToUnconstrained();
		return unconst;
	}
	public void setProb(int i, int j, IntegralParameters constrainedParameters, IntegralParameters unconstrainedParameters, double prob) {
		int num = i*number_sweep_points+j;
		if (i<5) {
			constrainedFileArray[num][0] = constrainedParameters.stream_parameters[0][i];
			constrainedFileArray[num][1] = prob;
			unconstrainedFileArray[num][0] = unconstrainedParameters.stream_parameters[0][i];
			unconstrainedFileArray[num][1] = prob;
		}
		else if (i==5) {
			constrainedFileArray[num][0] = constrainedParameters.stream_weights[0];
			constrainedFileArray[num][1] = prob;
			unconstrainedFileArray[num][0] = unconstrainedParameters.stream_weights[0];
			unconstrainedFileArray[num][1] = prob;
		}
		else if ((i>5)&&(i<number_parameters)) {
			constrainedFileArray[num][0] = constrainedParameters.background_parameters[i-6];
			constrainedFileArray[num][1] = prob;
			unconstrainedFileArray[num][0] = unconstrainedParameters.background_parameters[i-6];
			unconstrainedFileArray[num][1] = prob;
		}
	}
	public void makeParamSweepFile() {
		File cfile = new File("parameterSweepConstrained.txt");
		File ufile = new File("parameterSweepUnconstrained.txt");
		try {
			FileOutputStream cfos = new FileOutputStream(cfile);
			DataOutputStream coutfile = new DataOutputStream(cfos);
			FileOutputStream ufos = new FileOutputStream(ufile);
			DataOutputStream uoutfile = new DataOutputStream(ufos);
			for (int i = 0; i<number_sweep_points*number_parameters; i++){
				coutfile.writeBytes(constrainedFileArray[i][0]+" "+constrainedFileArray[i][1]+"\n");
				uoutfile.writeBytes(unconstrainedFileArray[i][0]+" "+unconstrainedFileArray[i][1]+"\n");
			}
			coutfile.close();
			uoutfile.close();
			System.err.println("Output written to: "+cfile+" and "+ufile);
		}
		catch (Exception e) {
			System.err.println("Error printing to file "+e);
			e.printStackTrace();
			System.exit(0);
		}

	}
}