package gmle.salsa.framework;

// Import declarations generated by the SALSA compiler, do not modify.
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import salsa_lite.core.language.Actor;
import salsa_lite.core.language.ActorReference;
import salsa_lite.core.language.IdentifierGenerator;
import salsa_lite.core.language.Message;
import salsa_lite.core.language.Token;
import salsa_lite.core.language.exceptions.CurrentContinuationException;
import salsa_lite.core.language.exceptions.MessageHandlerNotFoundException;

import salsa_lite.core.language.LiteActorState;

import salsa_lite.core.naming.UAN;
import salsa_lite.core.naming.UAL;
import salsa_lite.core.naming.MalformedUALException;
import salsa_lite.core.naming.MalformedUANException;

// End SALSA compiler generated import delcarations.

import java.lang.reflect.Constructor;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;

public class GeneticSearchProbabilisticState extends LiteActorState  {
	public GeneticSearchProbabilisticState(UAN uan) {
		super(uan);
		self = new GeneticSearchProbabilistic(getUAN());
	}

	public GeneticSearchProbabilisticState(UAL ual) {
		super(ual);
		self = new GeneticSearchProbabilistic(getUAL());
	}

	public GeneticSearchProbabilisticState(String identifier) {
		super(identifier);
		self = new GeneticSearchProbabilistic(getUAL());
	}

	public Object invokeMessage(String methodName, Object[] arguments) throws CurrentContinuationException, MessageHandlerNotFoundException, Exception {
		if (methodName.equals("construct") && arguments.length == 2 && (arguments[0] == null || arguments[0] instanceof String) && (arguments[1] == null || arguments[1] instanceof Integer)) {
			construct((String)arguments[0], ((Integer)arguments[1]).intValue());
			return null;
		}  else if (methodName.equals("construct") && arguments.length == 2 && (arguments[0] == null || arguments[0] instanceof String) && (arguments[1] == null || arguments[1] instanceof String)) {
			construct((String)arguments[0], (String)arguments[1]);
			return null;
		}  else if (methodName.equals("initialize") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof Object[])) {
			initialize((Object[])arguments[0]);
			return null;
		}  else if (methodName.equals("dummy") && arguments.length == 0) {
			dummy();
			return null;
		}  else if (methodName.equals("genetic_search") && arguments.length == 5 && (arguments[0] == null || arguments[0] instanceof double[]) && (arguments[1] == null || arguments[1] instanceof double[]) && (arguments[2] == null || arguments[2] instanceof Integer) && (arguments[3] == null || arguments[3] instanceof Long) && (arguments[4] == null || arguments[4] instanceof Double)) {
			genetic_search((double[])arguments[0], (double[])arguments[1], ((Integer)arguments[2]).intValue(), ((Long)arguments[3]).longValue(), ((Double)arguments[4]).doubleValue());
			return null;
		}  else if (methodName.equals("process_results") && arguments.length == 4 && (arguments[0] == null || arguments[0] instanceof Double) && (arguments[1] == null || arguments[1] instanceof double[]) && (arguments[2] == null || arguments[2] instanceof String) && (arguments[3] == null || arguments[3] instanceof Long)) {
			process_results(((Double)arguments[0]).doubleValue(), (double[])arguments[1], (String)arguments[2], ((Long)arguments[3]).longValue());
			return null;
		}  else if (methodName.equals("generate_next") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof Integer)) {
			generate_next(((Integer)arguments[0]).intValue());
			return null;
		}  else if (methodName.equals("get_random_parameters") && arguments.length == 0) {
			return get_random_parameters();
		}  else {
			return super.invokeMessage(methodName, arguments);
		}
	}

	long evaluations_done = 0;
	long max_evaluations;
	int population_size;
	double reproduction_chance;
	double[] parameter_min;
	double[] parameter_max;
	double[][] population;
	double[] population_likelihood;
	ActorReference[] evaluators;
	long start_time;
	long good_averages = 0;
	long bad_averages = 0;
	long good_mutations = 0;
	long bad_mutations = 0;
	long good_lower = 0;
	long bad_lower = 0;
	long good_higher = 0;
	long bad_higher = 0;
	public void construct(String eClass, int number_actors){
		evaluators = new ActorReference[number_actors];
				try {
			Method construct = null;
			Method[] methods = Class.forName(eClass).getDeclaredMethods();
			for (int i = 0; i<methods.length; i++){
				System.err.println("METHOD: "+methods[i]);
				if (methods[i].getName().equals("construct")) {
					construct = methods[i];
break;				}
			}
			for (int i = 0; i<number_actors; i++){
				evaluators[i] = (ActorReference)construct.invoke(null, new Object[]{ self, null, null, -1, new Object[0] });
			}
		}
		catch (Exception e) {
			System.err.println("GeneticSearchProbabilistic Error:");
			System.err.println("\tCould not create actor: "+eClass);
			System.err.println("\tThrew exception: "+e);
			e.printStackTrace();
		}

	}
	public void construct(String eClass, String theaters_file){
		ActorLocations actor_locations = new ActorLocations(theaters_file);
		evaluators = new ActorReference[actor_locations.getTotalActors()];
				try {
			Method construct = null;
			Method[] methods = Class.forName(eClass).getDeclaredMethods();
			for (int i = 0; i<methods.length; i++){
				System.err.println("METHOD: "+methods[i]);
				if (methods[i].getName().equals("construct")) {
					construct = methods[i];
break;				}
			}
			int current = 0;
			String[] theaters = actor_locations.getTheaters();
			int[] actors_per_theater = actor_locations.getActorsPerTheater();
			for (int i = 0; i<theaters.length; i++){
				for (int j = 0; j<actors_per_theater[i]; j++){
					UAN luan = new UAN(actor_locations.getNameServer()+"likelihood_evaluator_"+current);
					UAL lual = new UAL(theaters[i]+"likelihood_evaluator_"+current);
					evaluators[current] = (ActorReference)construct.invoke(null, new Object[]{ self, luan, lual.getHost(), lual.getPort(), new Object[0] });
					current++;
				}
			}
		}
		catch (Exception e) {
			System.err.println("GeneticSearchProbabilistic Error:");
			System.err.println("\tCould not create actor: "+eClass);
			System.err.println("\tThrew exception: "+e);
			e.printStackTrace();
		}

	}
	public void initialize(Object[] arguments) {
		System.out.println("initializing");
		{
			Token token_2_0 = new Token();
			// join block
			token_2_0.setJoinDirector();
			for (int i = 0; i<evaluators.length; i++){
				{
					// evaluators[i]<-initialize(arguments, 0, 1)
					{
						Message __message = new Message( self, evaluators[i], "initialize", new Object[]{ arguments, new Integer(0), new Integer(1) }, null, token_2_0);
						__messages.add( __message );
					}
				}
			}
			token_2_0.createJoinDirector();
			// dummy()
			{
				Message __message = new Message( self, self, "dummy", new Object[]{  }, token_2_0, currentMessage.getContinuationToken());
				__message.setJoinPosition(currentMessage.getJoinPosition());
				__messages.add( __message );
			}
			throw new CurrentContinuationException();
		}
	}
	public void dummy() {
	}
	public void genetic_search(double[] parameter_min, double[] parameter_max, int population_size, long max_evaluations, double reproduction_chance) {
		this.parameter_min = parameter_min;
		this.parameter_max = parameter_max;
		this.population_size = population_size;
		this.reproduction_chance = reproduction_chance;
		this.max_evaluations = max_evaluations;
		this.population = new double[0][parameter_min.length];
		this.population_likelihood = new double[0];
		start_time = System.currentTimeMillis();
		for (int i = 0; i<evaluators.length; i++){
			double[] params = this.get_random_parameters();
			{
				Token token_3_0 = new Token();
				Token token_3_1 = new Token();
				// evaluators[i]<-evaluate(params)
				{
					Message __message = new Message( self, evaluators[i], "evaluate", new Object[]{ params }, null, token_3_0);
					__messages.add( __message );
				}
				// process_results(token, params, "random", System.currentTimeMillis())
				{
					Message __message = new Message( self, self, "process_results", new Object[]{ token_3_0, params, "random", System.currentTimeMillis() }, token_3_0, token_3_1);
					__messages.add( __message );
				}
				// generate_next(new Integer(i))
				{
					Message __message = new Message( self, self, "generate_next", new Object[]{ new Integer(i) }, token_3_1, null);
					__messages.add( __message );
				}
			}
		}
	}
	public void process_results(double result, double[] parameters, String type, long send_time) {
		if (Double.isNaN(result)) {
			evaluations_done++;
			System.out.println("evaluations done: "+evaluations_done+", min: "+population_likelihood[0]+", max: "+population_likelihood[population_likelihood.length-1]+", median: "+population_likelihood[population_likelihood.length/2]+", current: "+result+" (NaN)");
			return;
		}
		boolean was_good = false;
		if (population.length==0) {
			population = new double[1][];
			population_likelihood = new double[1];
			population[0] = parameters;
			population_likelihood[0] = result;
		}
		else {
			int position = population.length;
			for (int i = 0; i<population.length; i++){
				if (result<population_likelihood[i]) {
					position = i;
break;				}
				else if (result==population_likelihood[i]) {
					evaluations_done++;
					System.out.println("evaluations done: "+evaluations_done+", min: "+population_likelihood[0]+", max: "+population_likelihood[population_likelihood.length-1]+", median: "+population_likelihood[population_likelihood.length/2]+", current: "+result+" (duplicate)");
					return;
				}
			}
			int population_length = Math.min(population_size, population.length+1);
			if (position<population_length) {
				double[][] population_temp = new double[population_length][];
				double[] population_likelihood_temp = new double[population_length];
				System.arraycopy(population, 0, population_temp, 0, position);
				System.arraycopy(population_likelihood, 0, population_likelihood_temp, 0, position);
				population_temp[position] = parameters;
				population_likelihood_temp[position] = result;
				int remaining = population_length-(position+1);
				System.arraycopy(population, position, population_temp, position+1, remaining);
				System.arraycopy(population_likelihood, position, population_likelihood_temp, position+1, remaining);
				population = population_temp;
				population_likelihood = population_likelihood_temp;
				if (type.equals("mutation")) {
					good_mutations++;
				}
				else if (type.equals("average")) {
					good_averages++;
				}
				else if (type.equals("lower")) {
					good_lower++;
				}
				else if (type.equals("higher")) {
					good_higher++;
				}
				was_good = true;
			}
			else {
				if (type.equals("mutation")) {
					bad_mutations++;
				}
				else if (type.equals("average")) {
					bad_averages++;
				}
				else if (type.equals("lower")) {
					bad_lower++;
				}
				else if (type.equals("higher")) {
					bad_higher++;
				}
			}
		}
		evaluations_done++;
		double evals_per_sec = ((double)evaluations_done)/(((double)System.currentTimeMillis()-(double)start_time)/1000.0);
		double mutation_rate = 0;
		double average_rate = 0;
		double lower_rate = 0;
		double higher_rate = 0;
		if (good_mutations+bad_mutations>0) mutation_rate = ((double)good_mutations)/((double)good_mutations+(double)bad_mutations);
		if (good_averages+bad_averages>0) average_rate = ((double)good_averages)/((double)good_averages+(double)bad_averages);
		if (good_lower+bad_lower>0) lower_rate = ((double)good_lower)/((double)good_lower+(double)bad_lower);
		if (good_higher+bad_higher>0) higher_rate = ((double)good_higher)/((double)good_higher+(double)bad_higher);
		System.out.println("evaluations done: "+evaluations_done+", min: "+population_likelihood[0]+", max: "+population_likelihood[population_likelihood.length-1]+", median: "+population_likelihood[population_likelihood.length/2]+", current: "+result+", evals/sec: "+evals_per_sec+", mutate: "+mutation_rate+", average: "+average_rate+", lower: "+lower_rate+", higher: "+higher_rate+", calc time: "+(System.currentTimeMillis()-send_time)+", good? "+was_good);
	}
	public void generate_next(int evaluator) {
		if (evaluations_done>=max_evaluations) 		return;
		if (population.length<population_size) {
			double[] params = this.get_random_parameters();
			{
				Token token_3_0 = new Token();
				Token token_3_1 = new Token();
				// evaluators[evaluator]<-evaluate(params)
				{
					Message __message = new Message( self, evaluators[evaluator], "evaluate", new Object[]{ params }, null, token_3_0);
					__messages.add( __message );
				}
				// process_results(token, params, "random", System.currentTimeMillis())
				{
					Message __message = new Message( self, self, "process_results", new Object[]{ token_3_0, params, "random", System.currentTimeMillis() }, token_3_0, token_3_1);
					__messages.add( __message );
				}
				// generate_next(new Integer(evaluator))
				{
					Message __message = new Message( self, self, "generate_next", new Object[]{ new Integer(evaluator) }, token_3_1, null);
					__messages.add( __message );
				}
			}
		}
		else {
			if (Math.random()<reproduction_chance) {
				int first = (int)(Math.random()*population_size);
				int second = (int)(Math.random()*population_size);
				double[] first_params = population[first];
				double[] second_params = population[second];
				double[] average_params = new double[first_params.length];
				double[] prev_params = new double[first_params.length];
				double[] next_params = new double[first_params.length];
				for (int i = 0; i<average_params.length; i++){
					average_params[i] = (first_params[i]+second_params[i])/2.0;
					double diff = average_params[i]-first_params[i];
					prev_params[i] = first_params[i]-diff;
					next_params[i] = second_params[i]+diff;
				}
				String prev, next;
				if (population_likelihood[first]<population_likelihood[second]) {
					prev = "lower";
					next = "higher";
				}
				else {
					prev = "higher";
					next = "lower";
				}
				{
					Token token_4_0 = new Token();
					// evaluators[evaluator]<-evaluate(average_params)
					{
						Message __message = new Message( self, evaluators[evaluator], "evaluate", new Object[]{ average_params }, null, token_4_0);
						__messages.add( __message );
					}
					// process_results(token, average_params, "average", System.currentTimeMillis())
					{
						Message __message = new Message( self, self, "process_results", new Object[]{ token_4_0, average_params, "average", System.currentTimeMillis() }, token_4_0, null);
						__messages.add( __message );
					}
				}
				{
					Token token_4_0 = new Token();
					// evaluators[evaluator]<-evaluate(prev_params)
					{
						Message __message = new Message( self, evaluators[evaluator], "evaluate", new Object[]{ prev_params }, null, token_4_0);
						__messages.add( __message );
					}
					// process_results(token, prev_params, prev, System.currentTimeMillis())
					{
						Message __message = new Message( self, self, "process_results", new Object[]{ token_4_0, prev_params, prev, System.currentTimeMillis() }, token_4_0, null);
						__messages.add( __message );
					}
				}
				{
					Token token_4_0 = new Token();
					Token token_4_1 = new Token();
					// evaluators[evaluator]<-evaluate(next_params)
					{
						Message __message = new Message( self, evaluators[evaluator], "evaluate", new Object[]{ next_params }, null, token_4_0);
						__messages.add( __message );
					}
					// process_results(token, next_params, next, System.currentTimeMillis())
					{
						Message __message = new Message( self, self, "process_results", new Object[]{ token_4_0, next_params, next, System.currentTimeMillis() }, token_4_0, token_4_1);
						__messages.add( __message );
					}
					// generate_next(new Integer(evaluator))
					{
						Message __message = new Message( self, self, "generate_next", new Object[]{ new Integer(evaluator) }, token_4_1, null);
						__messages.add( __message );
					}
				}
			}
			else {
				double[] params = new double[parameter_min.length];
				int target = (int)(Math.random()*population_size);
				System.arraycopy(params, 0, population[target], 0, population[target].length);
				int random_parameter = (int)(Math.random()*parameter_min.length);
				params[random_parameter] = parameter_min[random_parameter]+((parameter_max[random_parameter]-parameter_min[random_parameter])*Math.random());
				{
					Token token_4_0 = new Token();
					Token token_4_1 = new Token();
					// evaluators[evaluator]<-evaluate(params)
					{
						Message __message = new Message( self, evaluators[evaluator], "evaluate", new Object[]{ params }, null, token_4_0);
						__messages.add( __message );
					}
					// process_results(token, params, "mutation", System.currentTimeMillis())
					{
						Message __message = new Message( self, self, "process_results", new Object[]{ token_4_0, params, "mutation", System.currentTimeMillis() }, token_4_0, token_4_1);
						__messages.add( __message );
					}
					// generate_next(new Integer(evaluator))
					{
						Message __message = new Message( self, self, "generate_next", new Object[]{ new Integer(evaluator) }, token_4_1, null);
						__messages.add( __message );
					}
				}
			}
		}
	}
	public double[] get_random_parameters() {
		double[] parameters = new double[parameter_min.length];
		for (int i = 0; i<parameters.length; i++){
			parameters[i] = parameter_min[i]+((parameter_max[i]-parameter_min[i])*Math.random());
		}
		return parameters;
	}
}