package gmle.salsa.framework;

// Import declarations generated by the SALSA compiler, do not modify.
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import salsa_lite.core.language.Actor;
import salsa_lite.core.language.ActorReference;
import salsa_lite.core.language.IdentifierGenerator;
import salsa_lite.core.language.Message;
import salsa_lite.core.language.Token;
import salsa_lite.core.language.exceptions.CurrentContinuationException;
import salsa_lite.core.language.exceptions.MessageHandlerNotFoundException;

import salsa_lite.core.language.LiteActorState;

import salsa_lite.core.naming.UAN;
import salsa_lite.core.naming.UAL;
import salsa_lite.core.naming.MalformedUALException;
import salsa_lite.core.naming.MalformedUANException;

// End SALSA compiler generated import delcarations.

import java.lang.reflect.Constructor;

public class MLEvaluatorState extends LiteActorState  {
	public MLEvaluatorState(UAN uan) {
		super(uan);
		self = new MLEvaluator(getUAN());
	}

	public MLEvaluatorState(UAL ual) {
		super(ual);
		self = new MLEvaluator(getUAL());
	}

	public MLEvaluatorState(String identifier) {
		super(identifier);
		self = new MLEvaluator(getUAL());
	}

	public Object invokeMessage(String methodName, Object[] arguments) throws CurrentContinuationException, MessageHandlerNotFoundException, Exception {
		if (methodName.equals("construct") && arguments.length == 2 && (arguments[0] == null || arguments[0] instanceof String) && (arguments[1] == null || arguments[1] instanceof Integer)) {
			construct((String)arguments[0], ((Integer)arguments[1]).intValue());
			return null;
		}  else if (methodName.equals("construct") && arguments.length == 2 && (arguments[0] == null || arguments[0] instanceof String) && (arguments[1] == null || arguments[1] instanceof String)) {
			construct((String)arguments[0], (String)arguments[1]);
			return null;
		}  else if (methodName.equals("initialize") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof Object[])) {
			initialize((Object[])arguments[0]);
			return null;
		}  else if (methodName.equals("evaluate") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof double[])) {
			return new Double(evaluate((double[])arguments[0]));
		}  else if (methodName.equals("combine") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof Object[])) {
			return combine((Object[])arguments[0]);
		}  else if (methodName.equals("dummy") && arguments.length == 0) {
			dummy();
			return null;
		}  else {
			return super.invokeMessage(methodName, arguments);
		}
	}

	ActorReference[] evaluators;
	public void construct(String eClass, int number_actors){
		evaluators = new ActorReference[number_actors];
				try {
			Method construct = null;
			Method[] methods = Class.forName(eClass).getDeclaredMethods();
			for (int i = 0; i<methods.length; i++){
				System.err.println("METHOD: "+methods[i]);
				if (methods[i].getName().equals("construct")) {
					construct = methods[i];
break;				}
			}
			for (int i = 0; i<number_actors; i++){
				evaluators[i] = (ActorReference)construct.invoke(null, new Object[]{ self, null, null, -1, new Object[0] });
			}
		}
		catch (Exception e) {
			System.err.println("MLEvaluator Error:");
			System.err.println("\tCould not create actor: "+eClass);
			System.err.println("\tThrew exception: "+e);
			e.printStackTrace();
		}

	}
	public void construct(String eClass, String theaters_file){
		ActorLocations actor_locations = new ActorLocations(theaters_file);
		evaluators = new ActorReference[actor_locations.getTotalActors()];
				try {
			Method construct = null;
			Method[] methods = Class.forName(eClass).getDeclaredMethods();
			for (int i = 0; i<methods.length; i++){
				System.err.println("METHOD: "+methods[i]);
				if (methods[i].getName().equals("construct")) {
					construct = methods[i];
break;				}
			}
			int current = 0;
			String[] theaters = actor_locations.getTheaters();
			int[] actors_per_theater = actor_locations.getActorsPerTheater();
			for (int i = 0; i<theaters.length; i++){
				for (int j = 0; j<actors_per_theater[i]; j++){
					UAN luan = new UAN(actor_locations.getNameServer()+"likelihood_evaluator_"+current);
					UAL lual = new UAL(theaters[i]+"likelihood_evaluator_"+current);
					evaluators[current] = (ActorReference)construct.invoke(null, new Object[]{ self, luan, lual.getHost(), lual.getPort(), new Object[0] });
					current++;
				}
			}
		}
		catch (Exception e) {
			System.err.println("MLEvaluator Error:");
			System.err.println("\tCould not create actor: "+eClass);
			System.err.println("\tThrew exception: "+e);
			e.printStackTrace();
		}

	}
	public void initialize(Object[] arguments) {
		{
			Token token_2_0 = new Token();
			// join block
			token_2_0.setJoinDirector();
			for (int i = 0; i<evaluators.length; i++){
				{
					// evaluators[i]<-initialize(arguments, i, evaluators.length)
					{
						Message __message = new Message( self, evaluators[i], "initialize", new Object[]{ arguments, i, evaluators.length }, null, token_2_0);
						__messages.add( __message );
					}
				}
			}
			token_2_0.createJoinDirector();
			// dummy()
			{
				Message __message = new Message( self, self, "dummy", new Object[]{  }, token_2_0, currentMessage.getContinuationToken());
				__message.setJoinPosition(currentMessage.getJoinPosition());
				__messages.add( __message );
			}
			throw new CurrentContinuationException();
		}
	}
	public double evaluate(double[] parameters) {
		if (evaluators[0] instanceof IntegralEvaluator) {
			Token integrals = new Token("integrals");
			{
				Token token_3_0 = new Token();
				Token token_3_1 = new Token();
				// join block
				token_3_0.setJoinDirector();
				for (int i = 0; i<evaluators.length; i++){
					{
						// evaluators[i]<-evaluate_integrals(parameters)
						{
							Message __message = new Message( self, evaluators[i], "evaluate_integrals", new Object[]{ parameters }, null, token_3_0);
							__messages.add( __message );
						}
					}
				}
				token_3_0.createJoinDirector();
				// combine(token)
				{
					Message __message = new Message( self, self, "combine", new Object[]{ token_3_0 }, token_3_0, token_3_1);
					__messages.add( __message );
				}
				// token integrals = evaluators[0]<-combine_integrals(token)
				{
					Message __message = new Message( self, evaluators[0], "combine_integrals", new Object[]{ token_3_1 }, token_3_1, integrals);
					__messages.add( __message );
				}
			}
			{
				Token token_3_0 = new Token();
				Token token_3_1 = new Token();
				// join block
				token_3_0.setJoinDirector();
				for (int i = 0; i<evaluators.length; i++){
					{
						// evaluators[i]<-evaluate_likelihood(parameters, integrals)
						{
							Message __message = new Message( self, evaluators[i], "evaluate_likelihood", new Object[]{ parameters, integrals }, null, token_3_0);
							__messages.add( __message );
						}
					}
				}
				token_3_0.createJoinDirector();
				// combine(token)
				{
					Message __message = new Message( self, self, "combine", new Object[]{ token_3_0 }, token_3_0, token_3_1);
					__messages.add( __message );
				}
				// evaluators[0]<-combine_likelihood(token)
				{
					Message __message = new Message( self, evaluators[0], "combine_likelihood", new Object[]{ token_3_1 }, token_3_1, currentMessage.getContinuationToken());
					__message.setJoinPosition(currentMessage.getJoinPosition());
					__messages.add( __message );
				}
				throw new CurrentContinuationException();
			}
		}
		else {
			{
				Token token_3_0 = new Token();
				Token token_3_1 = new Token();
				// join block
				token_3_0.setJoinDirector();
				for (int i = 0; i<evaluators.length; i++){
					{
						// evaluators[i]<-evaluate_slice(parameters, i, evaluators.length)
						{
							Message __message = new Message( self, evaluators[i], "evaluate_slice", new Object[]{ parameters, i, evaluators.length }, null, token_3_0);
							__messages.add( __message );
						}
					}
				}
				token_3_0.createJoinDirector();
				// combine(token)
				{
					Message __message = new Message( self, self, "combine", new Object[]{ token_3_0 }, token_3_0, token_3_1);
					__messages.add( __message );
				}
				// evaluators[0]<-compose(token)
				{
					Message __message = new Message( self, evaluators[0], "compose", new Object[]{ token_3_1 }, token_3_1, currentMessage.getContinuationToken());
					__message.setJoinPosition(currentMessage.getJoinPosition());
					__messages.add( __message );
				}
				throw new CurrentContinuationException();
			}
		}
	}
	public double[][] combine(Object[] results) {
		double[][] dresults = new double[results.length][];
		for (int i = 0; i<results.length; i++){
			dresults[i] = (double[])results[i];
		}
		return dresults;
	}
	public void dummy() {
	}
}