package gmle.salsa.framework;


public class Matrix implements java.io.Serializable {

	int x, y;
	public double[][] values;

	public Matrix(int x, int y) {
		this.x = x;
		this.y = y;
		values = new double[x][y];
		for (int i = 0; i < x; i++) {
			for (int j = 0; j < y; j++) {
				values[i][j] = 0;
			}
		}
	}

	public Matrix mul(double d) {
		Matrix result = new Matrix(x, y);
		for (int i = 0; i < x; i++) {
			for (int j = 0; j < y; j++) {
				result.values[i][j] = values[i][j]*d;
			}
		}
		return result;
	}

	public Matrix mul(Matrix m) {
		Matrix result = null;
		if (this.y == m.x) {
			result = new Matrix(this.x, m.y);
			for (int i = 0; i < this.x; i++) {
				for (int j = 0; j < m.y; j++) {
					for (int k = 0; k < this.y; k++) {
						result.values[i][j] += this.values[i][k]*m.values[k][j];
					}
				}
			}
		} else if (this.x == 1 && this.y == 1) {
			result = new Matrix(m.x, m.y);
			for (int i = 0; i < m.x; i++) {
				for (int j = 0; j < m.y; j++) {
					result.values[i][j] = m.values[i][j] * this.values[0][0];
				}
			}
		} else if (m.x == 1 && m.y == 1) {
			result = new Matrix(this.x, this.y);
			for (int i = 0; i < m.x; i++) {
				for (int j = 0; j < m.y; j++) {
					result.values[i][j] = this.values[i][j] * m.values[0][0];
				}
			}		
		} else {
			System.err.println("ERROR in matrix multiply, m1.y != m2.x");
			System.err.println("m1.x: " + this.x + ", m1.y: " + this.y);
			System.err.println("m2.x: " + m.x + ", m2.y: " + m.y);
			System.exit(0);
		}
		return result;
	}

	Matrix add(Matrix m) {
		Matrix result = null;
		if (this.x == m.x && this.y == m.y) {
			result = new Matrix(this.x, this.y);
			for (int i = 0; i < this.x; i++) {
				for (int j = 0; j < this.y; j++) {
					result.values[i][j] = this.values[i][j] + m.values[i][j];
				}
			}
		} else {
			System.err.println("ERROR in matrix add, dimensions do not match!");
			System.exit(0);
		}
		return result;
	}

	Matrix sub(Matrix m) {
		Matrix result = null;
		if (this.x == m.x && this.y == m.y) {
			result = new Matrix(this.x, this.y);
			for (int i = 0; i < this.x; i++) {
				for (int j = 0; j < this.y; j++) {
					result.values[i][j] = this.values[i][j] - m.values[i][j];
				}
			}
		} else {
			System.err.println("ERROR in matrix sub, dimensions do not match!");
			System.exit(0);
		}
		return result;
	}

	Matrix rdiv(double d) {
		Matrix result = new Matrix(x, y);
		for (int i = 0; i < x; i++) {
			for (int j = 0; j < y; j++) {
				result.values[i][j] = values[i][j]/d;
			}
		}
		return result;
	}

	Matrix rdiv(Matrix m) {
		if (m.x == 1 && m.y == 1) {
			Matrix result = new Matrix(x, y);
			for (int i = 0; i < x; i++) {
				for (int j = 0; j < y; j++) {
					if (values[i][j] == 0 && m.values[0][0] == 0) {
						result.values[i][j] = 0;
					} else {
						result.values[i][j] = values[i][j]/m.values[0][0];
					}
				}
			}
			return result;
		} else if (m.x == m.y) {
			return this.mul(m.invert());
		} else {
			System.err.println("matrix_rdiv error: m2 not square");
			System.exit(0);
		}
		return null;
	}

	Matrix invert() {
		Matrix result = new Matrix(y,x);

		for (int i = 0; i < x; i++) {
			for (int j = 0; j < y; j++) {
				result.values[j][i] = values[i][j];
			}
		}
		return result;
	}

	Matrix neg() {
		Matrix result = new Matrix(x, y);
		for (int i = 0; i < x; i++) {
			for (int j = 0; j < y; j++) {
				result.values[i][j] = -values[i][j];
			}
		}
		return result;
	}

	double[] to_array() {
		if (y != 1) {
			System.err.println("ERROR matrix2array won't work because matrix.y != 1");
			System.exit(0);
		}

		double[] result = new double[x];
		for (int i = 0; i < x; i++) {
			result[i] = values[i][0];
		}
		return result;
	}

	double mean() {
		int count = 0;
		double total = 0.0;
		for (int i = 0; i < x; i++) {
			for (int j = 0; j < x; j++) {
				total += values[i][j];
				count++;
			}
		}
		return total/count;
	}

	double norm() {
		double result = 0.0;
		for (int i = 0; i < x; i++) {
			for (int j = 0; j < y; j++) {
				result += Math.sqrt(values[i][j]*values[i][j]);
			}
		}
		return result;
	}

	void print() {
		for (int i = 0; i < y; i++) {
			for (int j = 0; j < x; j++) {
				System.err.print(" " + values[j][i]);
			}
			System.err.println();
		}	
	}
}
