package gmle.salsa.physics;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.BufferedWriter;
import java.io.FileWriter;

import java.util.StringTokenizer;
import java.util.Vector;

import com.imsl.math.Complex;

public class LowMemoryArray implements java.io.Serializable {
	int x,y;
	Complex[] array;

	public Complex get(int target_x, int target_y) {
		try {
			return array[target_x*y + target_y];
		} catch (Exception e) {
			System.err.println("LowMemoryArray exception:");
			System.err.println("\tget: " + target_x + ", " + target_y);
			System.err.println("\tbounds: " + x + ", " + y);
			System.err.println("\tException: " + e);
			System.exit(0);
			return null;
		}
	}

	public void set(int target_x, int target_y, Complex value) {
		array[target_x*y + target_y] = value;
	}

	public Complex[] row(int x) {
		Complex[] column = new Complex[y];

		int position = 0;
		for (int i = y*x; i < (x+1)*y; i++) {
			column[position] = array[i];
			position++;
		}
		return column;
	}

	public Complex[] column(int y) {
		Complex[] row = new Complex[x];

		int position = 0;
		for (int i = y; i < x*this.y; i += this.y) {
			row[position] = array[i];
			position++;
		}
		return row;
	}

	public LowMemoryArray(int x, int y) {
		this.x = x;
		this.y = y;
		array = new Complex[x*y];
	}

	public LowMemoryArray(int x, int y, Complex[] array) {
		this.x = x;
		this.y = y;
		this.array = array;
	}

	public void writeToFile(String output_file) {
		System.err.println("writing data to file: " + output_file +", of size x: " + x + ", y: " + y);
		try {
			BufferedWriter out = new BufferedWriter(new FileWriter(output_file));
			out.write("array_size: " + x + ", " + y + "\n");
			for (int i = 0; i < x; i++) {
				for (int j = 0; j < y; j++) {
					out.write( get(i,j) + ", ");
				}
				out.write("\n");
			}
			out.flush();
			out.close();
		} catch (Exception e) {
			System.err.println("Error writing output file: " + output_file);
			System.err.println("\texception: " + e);
		}
	}


	public LowMemoryArray getData(int size, boolean right) {
		Complex[] keep_array = new Complex[(x-size)*y];
		Complex[] send_array = new Complex[size*y];

		if (right) {
			System.arraycopy(array, 0, keep_array, 0, array.length - y - (size*y));		//copy all but the last column + size columns into keep_array
			System.arraycopy(array, array.length - y - (size*y), send_array, 0, size*y);	//copy size columns into the send_array
			System.arraycopy(array, array.length-y, keep_array, keep_array.length-y, y);	//copy the last column into keep_array
		} else {
			System.arraycopy(array, 0, keep_array, 0, y);					//copy the first column into keep_array
			System.arraycopy(array, y, send_array, 0, send_array.length);			//copy size columns into send array
			System.arraycopy(array, y+send_array.length, keep_array, y, keep_array.length-y);	//copy the remaining into keep_array
		}

		this.array = keep_array;
		this.x = this.x-size;

		return new LowMemoryArray(size, y, send_array);
	}

	public void recieveData(LowMemoryArray recieved, boolean right) {
		if (y != recieved.y) {
			System.err.println("Error, invalid data returned, recieved.y != y");
			System.exit(0);
		}

		Complex[] new_array = new Complex[array.length + recieved.array.length];
		if (right) {
			System.arraycopy(array, 0, new_array, 0, array.length-y);					//copy all but the last column
			System.arraycopy(recieved.array, 0, new_array, array.length-y, recieved.array.length);		//copy the recieved data into this array
			System.arraycopy(array, array.length-y, new_array, array.length-y+recieved.array.length, y);	//copy the last row into the new array
		} else {
			System.arraycopy(array, 0, new_array, 0, y);							//copy the first column
			System.arraycopy(recieved.array, 0, new_array, y, recieved.array.length);			//copy the recieved array
			System.arraycopy(array, y, new_array, y+recieved.array.length, array.length-y);			//copy the remaineder of original array
		}

		x = x + recieved.x;
		array = new_array;
	}


	LowMemoryArray[] split(int number_slices) {
		/********
			*	Split over the y dimension
		*********/

		double slice_size = (double)(y)/(double)number_slices;
		double extra = slice_size%1.0;
		double carry = extra;

		System.err.println("splitting LowMemoryArray of x: " + x + ", y: " + y + ", into " + number_slices + " slices of size: " + x + ", " + slice_size);
		LowMemoryArray[] slices = new LowMemoryArray[number_slices];

		int total_y = 0;
		int slice_y_min = 0;

		for (int i = 0; i < number_slices; i++) {
			System.err.println("carry: " + carry);

			if (i == (number_slices-1) && carry > 0.99999) carry = 1.0;
			int slice_y_max = (int)((slice_size*(i+1)) + carry);

			System.err.println("creating slice from y: " + slice_y_min + " to " + slice_y_max + ", over x: " + x);

			slices[i] = new LowMemoryArray(x, slice_y_max-slice_y_min);
			for (int i_x = 0; i_x < x; i_x++) {
				for (int i_y = slice_y_min; i_y < slice_y_max; i_y++) {
					slices[i].set(i_x, (i_y-slice_y_min), get(i_x, i_y));
				}
			}

			slice_y_min = slice_y_max;
			carry += extra;
			if (carry >= 1.0) carry -= 1.0;

			total_y += slices[i].y;
		}
		System.err.println("Total y: " + total_y);

		return slices;
	}
}
