package gmle.salsa.physics;

// Import declarations generated by the SALSA compiler, do not modify.
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import salsa_lite.core.language.Actor;
import salsa_lite.core.language.ActorReference;
import salsa_lite.core.language.IdentifierGenerator;
import salsa_lite.core.language.Message;
import salsa_lite.core.language.Token;
import salsa_lite.core.language.exceptions.CurrentContinuationException;
import salsa_lite.core.language.exceptions.MessageHandlerNotFoundException;

import salsa_lite.core.language.LiteActorState;

import salsa_lite.core.naming.UAN;
import salsa_lite.core.naming.UAL;
import salsa_lite.core.naming.MalformedUALException;
import salsa_lite.core.naming.MalformedUANException;

// End SALSA compiler generated import delcarations.

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.Vector;
import java.util.StringTokenizer;
import gmle.salsa.framework.GradientDescent;
import gmle.salsa.framework.GeneticSearchProbabilistic;
import gmle.salsa.framework.Simplex;

public class MaxLikelihoodState extends LiteActorState  {
	public MaxLikelihoodState(UAN uan) {
		super(uan);
		self = new MaxLikelihood(getUAN());
	}

	public MaxLikelihoodState(UAL ual) {
		super(ual);
		self = new MaxLikelihood(getUAL());
	}

	public MaxLikelihoodState(String identifier) {
		super(identifier);
		self = new MaxLikelihood(getUAL());
	}

	public Object invokeMessage(String methodName, Object[] arguments) throws CurrentContinuationException, MessageHandlerNotFoundException, Exception {
		if (methodName.equals("act") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof String[])) {
			act((String[])arguments[0]);
			return null;
		}  else if (methodName.equals("construct") && arguments.length == 0) {
			construct();
			return null;
		} else {
			return super.invokeMessage(methodName, arguments);
		}
	}

	void construct() {}

	public void act(String[] arguments) {
		if (arguments.length!=3&&arguments.length!=4&&arguments.length!=5&&arguments.length!=6) {
			System.err.println("Invalid Arguments.");
			System.err.println("Usage:");
			System.err.println("\tjava physics_gml.MaxLikelihood <search_type> <data_file> <parameter_file>");
			System.err.println("\tjava physics_gml.MaxLikelihood <search_type> <data_file> <parameter_file>");
			System.err.println("Search Types:");
			System.err.println("\tsimplex = simplex");
			System.err.println("\tgd = gradient descent");
			System.err.println("\tcgd = conjugate gradient descent");
			System.err.println("\tgs = genetic search");
			System.err.println("\tgsp = genetic search probabilistic");
			System.exit(0);
		}
		String search_type = arguments[0];
		PhysicsParameters parameters = new PhysicsParameters(arguments[2]);
		PhysicsData data = new PhysicsData(arguments[1], parameters.get_pairs());
		String theaters_file = System.getProperty("theaters");
		parameters.print(System.out);
		if (search_type.equals("gsp")) {
			GeneticSearchProbabilistic gsp;
			if (theaters_file==null) {
				gsp = GeneticSearchProbabilistic.construct(self, null, null, -1, new Object[]{"gmle.salsa.physics.PhysicsEvaluator", 4});
			}
			else {
				gsp = GeneticSearchProbabilistic.construct(self, null, null, -1, new Object[]{"gmle.salsa.physics.PhysicsEvaluator", theaters_file});
			}
			{
				Token token_3_0 = new Token();
				// gsp<-initialize(new Object[]{ data, parameters })
				{
					Message __message = new Message( self, gsp, "initialize", new Object[]{ new Object[]{ data, parameters } }, null, token_3_0);
					__messages.add( __message );
				}
				// gsp<-genetic_search(parameters.get_parameter_min(), parameters.get_parameter_max(), 200, new Long(20000), 0.8)
				{
					Message __message = new Message( self, gsp, "genetic_search", new Object[]{ parameters.get_parameter_min(), parameters.get_parameter_max(), new Integer(200), new Long(20000), new Double(0.8) }, token_3_0, null);
					__messages.add( __message );
				}
			}
		}
		else if (search_type.equals("simplex")) {
			double[][] initial_parameters = parameters.get_simplex_parameters();
			Simplex simplex;
			if (theaters_file==null) {
				simplex = Simplex.construct(self, null, null, -1, new Object[]{"gmle.salsa.physics.PhysicsEvaluator", new Integer(4)});
			}
			else {
				simplex = Simplex.construct(self, null, null, -1, new Object[]{"gmle.salsa.physics.PhysicsEvaluator", theaters_file});
			}
			{
				Token token_3_0 = new Token();
				// simplex<-initialize(new Object[]{ data, parameters })
				{
					Message __message = new Message( self, simplex, "initialize", new Object[]{ new Object[]{ data, parameters } }, null, token_3_0);
					__messages.add( __message );
				}
				// simplex<-simplex(initial_parameters)
				{
					Message __message = new Message( self, simplex, "simplex", new Object[]{ initial_parameters }, token_3_0, null);
					__messages.add( __message );
				}
			}
		}
		else if (search_type.equals("cgd")||search_type.equals("gd")||search_type.equals("hcgd")) {
			double[] initial_parameters = parameters.get_double_points();
			double[] step_sizes = parameters.get_h();
			GradientDescent gradient_descent;
			if (theaters_file==null) {
				gradient_descent = GradientDescent.construct(self, null, null, -1, new Object[]{"gmle.salsa.physics.PhysicsEvaluator", new Integer(4)});
			}
			else {
				gradient_descent = GradientDescent.construct(self, null, null, -1, new Object[]{"gmle.salsa.physics.PhysicsEvaluator", theaters_file});
			}
			Token init_token = new Token("init_token");
			{
				// token init_token = gradient_descent<-initialize(new Object[]{ data, parameters })
				{
					Message __message = new Message( self, gradient_descent, "initialize", new Object[]{ new Object[]{ data, parameters } }, null, init_token);
					__messages.add( __message );
				}
			}
			if (arguments[0].equals("gd")) {
				{
					// gradient_descent<-gradient_descent(initial_parameters, step_sizes)
					{
						Message __message = new Message( self, gradient_descent, "gradient_descent", new Object[]{ initial_parameters, step_sizes }, null, null);
						Object[] _propertyInfo = { init_token };
						__message.setProperty( "waitfor", _propertyInfo );
						__messages.add( __message );
					}
				}
			}
			else if (arguments[0].equals("cgd")) {
				{
					// gradient_descent<-conjugate_gradient_descent(initial_parameters, step_sizes)
					{
						Message __message = new Message( self, gradient_descent, "conjugate_gradient_descent", new Object[]{ initial_parameters, step_sizes }, null, null);
						Object[] _propertyInfo = { init_token };
						__message.setProperty( "waitfor", _propertyInfo );
						__messages.add( __message );
					}
				}
			}
			else if (arguments[0].equals("hcgd")) {
				{
					// gradient_descent<-hessian_conjugate_gradient_descent(initial_parameters, step_sizes)
					{
						Message __message = new Message( self, gradient_descent, "hessian_conjugate_gradient_descent", new Object[]{ initial_parameters, step_sizes }, null, null);
						Object[] _propertyInfo = { init_token };
						__message.setProperty( "waitfor", _propertyInfo );
						__messages.add( __message );
					}
				}
			}
		}
		else {
			System.err.println("Search not implemented yet.");
			System.exit(0);
		}
	}
}