package gmle.salsa.physics;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.util.Vector;

import com.imsl.math.Complex;

public class PhysicsData {

	int number_streams;
	int stream_size;

	LowMemoryArray data;

	public int get_number_streams() {
		return number_streams;
	}

	public int get_stream_size() {
		return stream_size;
	}

	public Complex get(int x, int y) {
		return data.get(x,y);
	}

	/********
		*	For testing purposes.
	*********/
	public static void main(String[] arguments) {
		if (arguments.length != 2) {
			System.err.println("Invalid Arguments:");
			System.err.println("\tusage: java physics_gml.PhysicsData <parameters_file> <data_file>");
			System.exit(0);
		}

		PhysicsParameters test_parameters = new PhysicsParameters(arguments[0]);
		PhysicsData test_data = new PhysicsData(arguments[1], test_parameters.get_pairs());
	}

	public PhysicsData(String file, int[][][] pairs) {
		constructStreams(file, pairs);
	}

	private PhysicsData(LowMemoryArray slice) {
		this.data = slice;
		this.number_streams = data.x;
		this.stream_size = data.y;
	}

	public PhysicsData get_slice(int rank, int max_rank) {
		LowMemoryArray[] slices = data.split(max_rank);
		return new PhysicsData(slices[rank]);
	}
	

	public void constructStreams(String file, int[][][] pairs) {
		try {
			BufferedReader mapfile = new BufferedReader(new FileReader(file));
			String line;
			Vector mappings = new Vector();
			Vector usedids = new Vector();

			System.err.println("USEDIDS:");

			for (int i = 0; i < pairs.length; i++) {
				for (int j = 0; j < pairs[i].length; j++) {
					for (int k = 0; k < pairs[i][j].length; k++) {
						if (usedids.indexOf(new Integer(pairs[i][j][k])) == -1) {
							usedids.add(new Integer(pairs[i][j][k]));
							System.err.println("\t" + pairs[i][j][k]);
						}
					}
				}
			}

			StringTokenizer st = new StringTokenizer(mapfile.readLine(), ": ");
			if (!st.nextToken().equals("number_streams")) throw new Exception("Error reading number_streams");
			number_streams = Integer.parseInt(st.nextToken());

			st = new StringTokenizer(mapfile.readLine(), ": ");
			if (!st.nextToken().equals("stream_size")) throw new Exception("Error reading stream_size");
			stream_size = Integer.parseInt(st.nextToken());

			while ((line = mapfile.readLine()) != null) {
				st = new StringTokenizer(line);
				try {
					String filename = st.nextToken();
					Integer id = Integer.valueOf(st.nextToken());
					if (usedids.indexOf(id) != -1) {
						Vector mapping = new Vector();
						mapping.add(id);
						mapping.add(filename);
						mappings.add(mapping);
					}
				} catch (NoSuchElementException e) {
					System.err.println("Error parsing mappings!");
					throw e;
				}
			}

			data = new LowMemoryArray(number_streams, stream_size);

			ComplexStream[] complex_streams = new ComplexStream[mappings.size()];
			for (int i = 0; i < complex_streams.length; i++) {
				complex_streams[i] = new ComplexStream((String)((Vector)mappings.get(i)).get(1));

				System.err.print("stream[" + i + "], key: " + ((Vector)mappings.get(i)).get(0));
				int count = 0;
				while (true) {
					try {
						complex_streams[i].next();
						data.set(i, count, complex_streams[i].get());
						count++;
					} catch (Exception e) {
						break;
					}
				}
				System.err.println(", count: " + count);
			}
		} catch (Exception e) {
			System.err.println("Error in constructStreams: " + e);
		}
	}
}

