package gmle.salsa.physics;

// Import declarations generated by the SALSA compiler, do not modify.
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import salsa_lite.core.language.Actor;
import salsa_lite.core.language.ActorReference;
import salsa_lite.core.language.IdentifierGenerator;
import salsa_lite.core.language.Message;
import salsa_lite.core.language.Token;
import salsa_lite.core.language.exceptions.CurrentContinuationException;
import salsa_lite.core.language.exceptions.MessageHandlerNotFoundException;

import salsa_lite.core.language.LiteActorState;

import salsa_lite.core.naming.UAN;
import salsa_lite.core.naming.UAL;
import salsa_lite.core.naming.MalformedUALException;
import salsa_lite.core.naming.MalformedUANException;

// End SALSA compiler generated import delcarations.

import com.imsl.math.Complex;
import gmle.salsa.framework.Evaluator;
import gmle.salsa.framework.EvaluatorState;

public class PhysicsEvaluatorState extends LiteActorState implements EvaluatorState {
	public PhysicsEvaluatorState(UAN uan) {
		super(uan);
		self = new PhysicsEvaluator(getUAN());
	}

	public PhysicsEvaluatorState(UAL ual) {
		super(ual);
		self = new PhysicsEvaluator(getUAL());
	}

	public PhysicsEvaluatorState(String identifier) {
		super(identifier);
		self = new PhysicsEvaluator(getUAL());
	}

	public Object invokeMessage(String methodName, Object[] arguments) throws CurrentContinuationException, MessageHandlerNotFoundException, Exception {
		if (methodName.equals("act") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof String[])) {
			act((String[])arguments[0]);
			return null;
		}  else if (methodName.equals("initialize") && arguments.length == 3 && (arguments[0] == null || arguments[0] instanceof Object[]) && (arguments[1] == null || arguments[1] instanceof Integer) && (arguments[2] == null || arguments[2] instanceof Integer)) {
			initialize((Object[])arguments[0], ((Integer)arguments[1]).intValue(), ((Integer)arguments[2]).intValue());
			return null;
		}  else if (methodName.equals("set_complex_points") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof double[])) {
			set_complex_points((double[])arguments[0]);
			return null;
		}  else if (methodName.equals("compose") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof double[][])) {
			return new Double(compose((double[][])arguments[0]));
		}  else if (methodName.equals("evaluate_slice") && arguments.length == 3 && (arguments[0] == null || arguments[0] instanceof double[]) && (arguments[1] == null || arguments[1] instanceof Integer) && (arguments[2] == null || arguments[2] instanceof Integer)) {
			return evaluate_slice((double[])arguments[0], ((Integer)arguments[1]).intValue(), ((Integer)arguments[2]).intValue());
		}  else if (methodName.equals("evaluate") && arguments.length == 1 && (arguments[0] == null || arguments[0] instanceof double[])) {
			return new Double(evaluate((double[])arguments[0]));
		}  else if (methodName.equals("correct") && arguments.length == 0) {
			correct();
			return null;
		}  else if (methodName.equals("get_result") && arguments.length == 0) {
			get_result();
			return null;
		}  else if (methodName.equals("calculate_correction") && arguments.length == 0) {
			return new Double(calculate_correction());
		}  else if (methodName.equals("construct") && arguments.length == 0) {
			construct();
			return null;
		} else {
			return super.invokeMessage(methodName, arguments);
		}
	}

	void construct() {}

	public void act(String[] arguments) {
		if (arguments.length!=2) {
			System.err.println("Invalid Arguments");
			System.err.println("\tusage: java physics_gml.PhysicsFunction <parameter_file> <data_file>\n");
			System.exit(0);
		}
		PhysicsParameters parameters = new PhysicsParameters(arguments[0]);
		data = new PhysicsData(arguments[1], parameters.get_pairs());
		complex_points = parameters.get_complex_points();
		psis = parameters.get_psis();
		pairs = parameters.get_pairs();
		this.evaluate(parameters.get_double_points());
		PhysicsResults[] test_results = new PhysicsResults[1];
		System.err.println("Result: "+current_result);
	}
	PhysicsData data;
	Complex[] complex_points;
	Complex[][] psis;
	int[][][] pairs;
	public void initialize(Object[] arguments, int rank, int max_rank) {
		this.data = (PhysicsData)arguments[0];
		this.pairs = ((PhysicsParameters)arguments[1]).get_pairs();
		this.psis = ((PhysicsParameters)arguments[1]).get_psis();
		this.data = this.data.get_slice(rank, max_rank);
		System.err.println("initialized data");
	}
	double current_result;
	double correct_result;
	int number_events;
	public void set_complex_points(double[] parameters) {
		complex_points = new Complex[parameters.length/2];
		for (int i = 0; i<complex_points.length; i++){
			complex_points[i] = new Complex(parameters[2*i], parameters[(2*i)+1]);
		}
	}
	public double compose(double[][] result_slices) {
		this.current_result = 0;
		for (int i = 0; i<result_slices.length; i++){
			this.current_result += result_slices[i][0];
			this.number_events += (int)result_slices[i][1];
		}
		this.correct();
		return correct_result;
	}
	public double[] evaluate_slice(double[] parameters, int rank, int max_rank) {
		this.set_complex_points(parameters);
		this.get_result();
		return new double[]{ this.current_result, this.number_events };
	}
	public double evaluate(double[] parameters) {
		this.set_complex_points(parameters);
		this.get_result();
		this.correct();
		return correct_result;
	}
	public void correct() {
		double correction = this.calculate_correction();
		correct_result = -1.0*(current_result-(number_events*correction));
	}
	public void get_result() {
		current_result = 0;
		number_events = 0;
		for (int i = 0; i<data.get_stream_size(); i++){
			number_events++;
			Complex step_result = new Complex();
			for (int blockNum = 0; blockNum<pairs.length; blockNum++){
				Complex block_result = new Complex();
				int[][] block = pairs[blockNum];
				for (int pairNum = 0; pairNum<block.length; pairNum++){
					int[] pair = block[pairNum];
					Complex arg = complex_points[pair[0]];
					Complex point = data.get(pair[1], i);
					block_result = block_result.plus(arg.times(point));
				}
				step_result = step_result.plus(block_result.times(Complex.conjugate(block_result)));
			}
			if (step_result.imag()>1) {
				System.err.println("Warning: High imaginary component of step result! "+step_result);
			}
			current_result += Math.log(step_result.real());
		}
	}
	public double calculate_correction() {
		Complex normalization = new Complex();
		for (int blockNum = 0; blockNum<pairs.length; blockNum++){
			int[][] block = pairs[blockNum];
			for (int pair1Num = 0; pair1Num<block.length; pair1Num++){
				int[] pair1 = block[pair1Num];
				for (int pair2Num = 0; pair2Num<block.length; pair2Num++){
					int[] pair2 = block[pair2Num];
					Complex temp = Complex.times(complex_points[pair1[0]], psis[pair1[1]][pair2[1]]);
					normalization = normalization.plus(Complex.times(temp, Complex.conjugate(complex_points[pair2[0]])));
				}
			}
		}
		if (normalization.imag()>1) System.err.println("Warning: high imaginary component of normalization: "+normalization);
		return normalization.real();
	}
}