package gmle.salsa.physics;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.OutputStream;

import java.util.StringTokenizer;

import com.imsl.math.Complex;

 public class PhysicsParameters {
	/********
		*	For testing purposes.
	*********/
	public static void main(String[] arguments) {
		PhysicsParameters test = new PhysicsParameters(arguments[0]);
		test.print(System.out);
	}

	public String toString() {
		String retval = "";
		for (int i = 0; i < complex_points.length; i++) {
			retval += complex_points[i] + " ";
		}
		return retval;
	}

	int monte_events;

	Complex[] complex_points;

	public Complex[] get_complex_points() {
		return complex_points;
	}

	public void set_complex_points(Complex[] complex_points) {
		this.complex_points = new Complex[complex_points.length];
		System.arraycopy(complex_points, 0, this.complex_points, 0, complex_points.length);
	}

	public void set_complex_points(double[] double_points) {
		complex_points = new Complex[double_points.length/2];
		for (int i = 0; i < double_points.length; i += 2) {
			complex_points[i/2] = new Complex(double_points[i], double_points[i+1]);
		}
	}

	public double[] get_double_points() {
		double[] double_points = new double[complex_points.length*2];
		for (int i = 0; i < complex_points.length; i++) {
			double_points[2*i] = complex_points[i].real();
			double_points[(2*i)+1] = complex_points[i].imag();
		}
		return double_points;
	}

	public double[] get_parameter_min() {
		double[] mins = new double[complex_points.length*2];
		for (int i = 0; i < complex_points.length*2; i++) {
			mins[i] = -50;
		}
		return mins;
	}

	public double[] get_parameter_max() {
		double[] maxs = new double[complex_points.length*2];
		for (int i = 0; i < complex_points.length*2; i++) {
			maxs[i] = 50;
		}
		return maxs;
	}

	public double[] get_h() {
		double[] h = new double[complex_points.length*2];
		for (int i = 0; i < h.length; i++) h[i] = 0.00001;
		return h;
	}

	public double[][] get_simplex_parameters() {
		double[] parameters = this.get_double_points();
		double[][] parameter_set = new double[parameters.length+1][parameters.length];

		System.arraycopy(parameters, 0, parameter_set[0], 0, parameters.length);

		for (int i = 0; i < parameter_set.length-1; i++) {
			parameters[i] += 1;
			System.arraycopy(parameters, 0, parameter_set[i+1], 0, parameters.length);
			parameters[i] -= 1;
		}

		System.err.println("generated initial simplex parameters:");
		for (int i = 0; i < parameter_set.length; i++) {
			System.err.print("[" + i + "]:");
			for (int j = 0; j < parameter_set[i].length; j++) System.err.print(" " + parameter_set[i][j]);
			System.err.println();
		}

		return parameter_set;
	}

	int[][][] pairs;

	public int[][][] get_pairs() {
		return pairs;
	}

	double min, max, h, mutation_range;

	Complex[][] psis;

	public Complex[][] get_psis() {
		return psis;
	}

	public double[] getH() {
		double[] hs = new double[complex_points.length * 2];
		for (int i = 0; i < hs.length; i++) hs[i] = h;
		return hs;
	}

	private PhysicsParameters() {}

	private PhysicsParameters(int[][][] pairs, Complex[][] psis) {
		this.pairs = new int[pairs.length][][];
		for (int i = 0; i < pairs.length; i++) {
			this.pairs[i] = new int[pairs[i].length][];
			for (int j = 0; j < pairs[i].length; j++) {
				this.pairs[i][j] = new int[pairs[i][j].length];
				System.arraycopy(pairs[i][j], 0, this.pairs[i][j], 0, pairs[i][j].length);
			}
		}

		this.psis = new Complex[psis.length][];
		for (int i = 0; i < psis.length; i++) {
			this.psis[i] = new Complex[psis[i].length];
			System.arraycopy(psis[i], 0, this.psis[i], 0, psis[i].length);
		}
	}

	public PhysicsParameters(String parameter_file) {
		readFromFile(parameter_file);
	}

	public PhysicsParameters(PhysicsParameters original) {
		this(original.get_pairs(), original.get_psis());

		this.monte_events = original.monte_events;
		this.set_complex_points(original.get_complex_points());
	}

	public PhysicsParameters copy() {
		return new PhysicsParameters(this);
	}

	public void readFromFile(String parameter_file) {
		int line_number = 0;
		try {
			BufferedReader parameterReader = new BufferedReader(new FileReader(parameter_file));

			/********
				*	Read the complex points
			*********/
			StringTokenizer st = new StringTokenizer(parameterReader.readLine(), " :"); line_number++;
			if (!st.nextToken().equals("number_complex_points")) throw new Exception("Error reading number_complex_points on line: " + line_number);
			complex_points = new Complex[Integer.parseInt(st.nextToken())];

			for (int i = 0; i < complex_points.length; i++) {
				st = new StringTokenizer(parameterReader.readLine(), ", "); line_number++;
				complex_points[i] = new Complex( Double.parseDouble(st.nextToken()), Double.parseDouble(st.nextToken()) );
			}

			parameterReader.readLine(); line_number++;

			/********
				*	Read the pairs
			*********/
			st = new StringTokenizer(parameterReader.readLine(), " :"); line_number++;
			if (!st.nextToken().equals("number_pair_blocks")) throw new Exception("Error reading number_pair_blocks on line: " + line_number);
			pairs = new int[Integer.parseInt(st.nextToken())][][];

			for (int i = 0; i < pairs.length; i++) {
				st = new StringTokenizer(parameterReader.readLine(), " :"); line_number++;
				if (!st.nextToken().equals("block_size")) throw new Exception("Error reading block_size on line: " + line_number);
				pairs[i] = new int[Integer.parseInt(st.nextToken())][2];

				for (int j = 0; j < pairs[i].length; j++) {
					st = new StringTokenizer(parameterReader.readLine(), ", "); line_number++;
					pairs[i][j][0] = Integer.parseInt(st.nextToken());
					pairs[i][j][1] = Integer.parseInt(st.nextToken());
				}
			}

			parameterReader.readLine(); line_number++;

			/********
				*	Read the psi matrix
			*********/
			st = new StringTokenizer(parameterReader.readLine(), ", :"); line_number++;
			if (!st.nextToken().equals("psi_matrix_size")) throw new Exception("Error reading psi_matrix_size on line: " + line_number);
			psis = new Complex[Integer.parseInt(st.nextToken())][Integer.parseInt(st.nextToken())];

			st = new StringTokenizer(parameterReader.readLine(), ", :"); line_number++;
			if (!st.nextToken().equals("monte_events")) throw new Exception("Error reading monte_events on line: " + line_number);
			monte_events = Integer.parseInt(st.nextToken());

			st = new StringTokenizer(parameterReader.readLine(), ", :"); line_number++;
			if (!st.nextToken().equals("h")) throw new Exception("Error reading h on line: " + line_number);
			h = Double.parseDouble(st.nextToken());

			st = new StringTokenizer(parameterReader.readLine(), ", :"); line_number++;
			if (!st.nextToken().equals("mutation_range")) throw new Exception("Error reading mutation_range on line: " + line_number);
			mutation_range = Double.parseDouble(st.nextToken());

			st = new StringTokenizer(parameterReader.readLine(), ", :"); line_number++;
			if (!st.nextToken().equals("min")) throw new Exception("Error reading min on line: " + line_number);
			min = Double.parseDouble(st.nextToken());

			st = new StringTokenizer(parameterReader.readLine(), ", :"); line_number++;
			if (!st.nextToken().equals("max")) throw new Exception("Error reading max on line: " + line_number);
			max = Double.parseDouble(st.nextToken());

			for (int i = 0; i < psis.length; i++) {
				st = new StringTokenizer(parameterReader.readLine(), ", "); line_number++;
				for (int j = 0; j < psis[i].length; j++) {
					psis[i][j] = Complex.valueOf(st.nextToken()).over(monte_events);
				}
			}

			parameterReader.close();
		} catch (Exception e) {
			System.err.println("Error reading parameter file: " + parameter_file + ", on line number: " + line_number);
			System.err.println("Exception: " + e);
		}
	}

	public void print(String file) {
		try {
			FileOutputStream out = new FileOutputStream(file);
			print(out);
			out.close();
		} catch(Exception e) {
			System.err.println("Error writing parameters to file: " + file);
			System.err.println("Exception: " + e);
		}
	}

	public void print(OutputStream outstream) {
		try {
			DataOutputStream out = new DataOutputStream(outstream);

			out.writeBytes("number_complex_points: " + complex_points.length + "\n");
			for (int i = 0; i < complex_points.length; i++) out.writeBytes(complex_points[i].real() + ", " + complex_points[i].imag() + "\n");
			out.writeBytes("\n");

			out.writeBytes("number_pair_blocks: " + pairs.length + "\n");
			for (int i = 0; i < pairs.length; i++) {
				out.writeBytes("block_size: " + pairs[i].length + "\n");
				for (int j = 0; j < pairs[i].length; j++) {
					out.writeBytes(pairs[i][j][0] + ", " + pairs[i][j][1] + "\n");
				}
			}
			out.writeBytes("\n");

			out.writeBytes("psi_matrix_size: " + psis.length + ", " + psis[0].length + "\n");
			out.writeBytes("monte_events: " + monte_events + "\n");
			for (int i = 0; i < psis.length; i++) {
				for (int j = 0; j < psis[i].length; j++) {
					out.writeBytes(psis[i][j].toString());
					if (j != psis[i].length-1) out.writeBytes(", ");
				}
				out.writeBytes("\n");
			}
		} catch (Exception e) {
			System.err.println("Error writing parameters to output stream: " + outstream);
			System.err.println("Exception: " + e);
		}
	}
}
